@@ -29,7 +29,6 @@
 #include <gtkmm/recentmanager.h>
 #include <gtkmm/filechooserdialog.h>
 #include <gtkmm/stock.h>
-#include <glibmm/miscutils.h>
 
 #ifdef GLOM_ENABLE_MAEMO
 #include <hildon-fmmm/file-chooser-dialog.h>
@@ -44,7 +43,7 @@
 #include <iostream>
 
 #ifdef GLOM_ENABLE_CLIENT_ONLY
-#define NEW_PAGE 1
+static const int NEW_PAGE = 1;
 #endif /* GLOM_ENABLE_CLIENT_ONLY */
 
 namespace
@@ -114,22 +113,21 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   builder->get_widget("existing_or_new_label", label);
   label->set_text(_("Open a Document"));
 #endif //GLOM_ENABLE_CLIENT_ONLY
-  
+
   builder->get_widget("existing_or_new_existing_treeview", m_existing_view);
   g_assert(m_existing_view);
 
   builder->get_widget("existing_or_new_notebook", m_notebook);
   builder->get_widget("existing_or_new_button_select", m_select_button);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
+
   if(!m_notebook || !m_select_button)
     throw std::runtime_error("Glade file does not contain the notebook or the select button for ExistingOrNew dialog.");
-#endif
 
   m_existing_model = Gtk::TreeStore::create(m_existing_columns);
   m_existing_model->set_sort_column(m_existing_columns.m_col_time, Gtk::SORT_DESCENDING);
   m_existing_view->set_model(m_existing_model);
 
- 
+
 
   m_iter_existing_other = m_existing_model->append();
   (*m_iter_existing_other)[m_existing_columns.m_col_title] = _("Select File");
@@ -141,8 +139,8 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
 
   m_iter_existing_recent = m_existing_model->append();
   (*m_iter_existing_recent)[m_existing_columns.m_col_title] = _("Recently Opened");
-  
- 
+
+
 
   m_existing_column_title.set_expand(true);
   m_existing_column_title.pack_start(m_existing_icon_renderer, false);
@@ -153,7 +151,92 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
 
   m_existing_view->set_headers_visible(false);
   m_existing_view->signal_row_activated().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_existing_row_activated));
- 
+
+
+  // Browse local network
+#ifndef G_OS_WIN32
+  gchar* service_type = epc_service_type_new(EPC_PROTOCOL_HTTPS, "glom");
+  m_service_monitor = epc_service_monitor_new_for_types(0, service_type, (void*)0);
+  g_signal_connect(m_service_monitor, "service-found", G_CALLBACK(on_service_found_static), this);
+  g_signal_connect(m_service_monitor, "service-removed", G_CALLBACK(on_service_removed_static), this);
+  g_free(service_type);
+#endif
+
+  // Add recently used files
+  Gtk::RecentManager::ListHandle_RecentInfos infos = Gtk::RecentManager::get_default()->get_items();
+  for(Gtk::RecentManager::ListHandle_RecentInfos::const_iterator iter = infos.begin(); iter != infos.end(); ++ iter)
+  {
+    Glib::RefPtr<Gtk::RecentInfo> info = *iter;
+    if(info->get_mime_type() == "application/x-glom")
+    {
+      Gtk::TreeModel::iterator iter = m_existing_model->append(m_iter_existing_recent->children());
+      (*iter)[m_existing_columns.m_col_title] = info->get_display_name();
+      (*iter)[m_existing_columns.m_col_time] = info->get_modified();
+      (*iter)[m_existing_columns.m_col_recent_info] = info;
+    }
+  }
+
+  const Gtk::TreeNodeChildren& children = m_iter_existing_recent->children();
+  if(children.begin() == children.end())
+    m_iter_existing_recent_dummy = create_dummy_item_existing(m_iter_existing_recent, _(RECENT_DUMMY_TEXT));
+
+#ifndef G_OS_WIN32
+  // Will be removed when items are added:
+  m_iter_existing_network_dummy = create_dummy_item_existing(m_iter_existing_network, _(NETWORK_DUMMY_TEXT));
+#endif
+
+
+  // Expand recently used files and the networked files,
+  // because the contents help to explain what this is:
+  m_existing_view->expand_row(m_existing_model->get_path(m_iter_existing_recent), false);
+#ifndef G_OS_WIN32
+  m_existing_view->expand_row(m_existing_model->get_path(m_iter_existing_network), false);
+#endif
+
+  m_select_button->signal_clicked().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_select_clicked));
+  m_select_button->set_image(*Gtk::manage(new Gtk::Image(Gtk::Stock::APPLY, Gtk::ICON_SIZE_BUTTON)));
+
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  m_notebook->signal_switch_page().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_switch_page));
+#endif /* !GLOM_ENABLE_CLIENT_ONLY */
+
+  Glib::RefPtr<Gtk::TreeView::Selection> existing_view_selection = m_existing_view->get_selection();
+  existing_view_selection->signal_changed().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_existing_selection_changed));
+  existing_view_selection->set_select_function( sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_existing_select_func) );
+
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  builder->get_widget("existing_or_new_new_treeview", m_new_view);
+  g_assert(m_new_view);
+  m_new_model = Gtk::TreeStore::create(m_new_columns);
+  m_new_view->set_model(m_new_model);
+
+  m_iter_new_empty = m_new_model->append();
+  (*m_iter_new_empty)[m_new_columns.m_col_title] = _("New Empty Document");
+
+  m_iter_new_template = m_new_model->append();
+  (*m_iter_new_template)[m_new_columns.m_col_title] = _("New From Template");
+
+  m_new_column_title.set_expand(true);
+  m_new_column_title.pack_start(m_new_icon_renderer, false);
+  m_new_column_title.pack_start(m_new_title_renderer, true);
+  m_new_column_title.set_cell_data_func(m_new_icon_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::new_icon_data_func));
+  m_new_column_title.set_cell_data_func(m_new_title_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::new_title_data_func));
+  m_new_view->append_column(m_new_column_title);
+
+  m_new_view->set_headers_visible(false);
+  m_new_view->signal_row_activated().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_row_activated));
+
+  m_iter_new_template_dummy = create_dummy_item_new(m_iter_new_template, _(TEMPLATE_DUMMY_TEXT));
+
+  Glib::RefPtr<Gtk::TreeView::Selection> new_view_selection = m_new_view->get_selection();
+  new_view_selection->signal_changed().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_selection_changed));
+  new_view_selection->set_select_function( sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_select_func) );
+#else /* GLOM_ENABLE_CLIENT_ONLY */
+  m_notebook->remove_page(NEW_PAGE);
+  m_notebook->set_show_tabs(false);
+#endif /* !GLOM_ENABLE_CLIENT_ONLY */
+
+
  // Load example files:
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 
@@ -177,94 +260,9 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
 
 #endif //!GLOM_ENABLE_CLIENT_ONLY
 
-
-  // Browse local network
-#ifndef G_OS_WIN32
-  gchar* service_type = epc_service_type_new(EPC_PROTOCOL_HTTPS, "glom");
-  m_service_monitor = epc_service_monitor_new_for_types(0, service_type, (void*)0);
-  g_signal_connect(m_service_monitor, "service-found", G_CALLBACK(on_service_found_static), this);
-  g_signal_connect(m_service_monitor, "service-removed", G_CALLBACK(on_service_removed_static), this);
-  g_free(service_type);
-#endif
-
-  // Add recently used files
-  typedef std::vector< Glib::RefPtr<Gtk::RecentInfo> > type_vec_infos;
-  type_vec_infos infos = Gtk::RecentManager::get_default()->get_items();
-  for(type_vec_infos::const_iterator iter = infos.begin(); iter != infos.end(); ++ iter)
-  {
-    Glib::RefPtr<Gtk::RecentInfo> info = *iter;
-    if(info->get_mime_type() == "application/x-glom")
-    {
-      Gtk::TreeModel::iterator iter = m_existing_model->append(m_iter_existing_recent->children());
-      (*iter)[m_existing_columns.m_col_title] = info->get_display_name();
-      (*iter)[m_existing_columns.m_col_time] = info->get_modified();
-      (*iter)[m_existing_columns.m_col_recent_info] = info;
-    }
-  }
-
-  const Gtk::TreeNodeChildren& children = m_iter_existing_recent->children();
-  if(children.begin() == children.end())
-    m_iter_existing_recent_dummy = create_dummy_item_existing(m_iter_existing_recent, _(RECENT_DUMMY_TEXT));
-
-#ifndef G_OS_WIN32
-  // Will be removed when items are added:
-  m_iter_existing_network_dummy = create_dummy_item_existing(m_iter_existing_network, _(NETWORK_DUMMY_TEXT));
-#endif
-
- 
-  // Expand recently used files and the networked files,
-  // because the contents help to explain what this is: 
-  m_existing_view->expand_row(m_existing_model->get_path(m_iter_existing_recent), false);
-#ifndef G_OS_WIN32
-  m_existing_view->expand_row(m_existing_model->get_path(m_iter_existing_network), false);
-#endif
-
-  m_select_button->signal_clicked().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_select_clicked));
-  m_select_button->set_image(*Gtk::manage(new Gtk::Image(Gtk::Stock::APPLY, Gtk::ICON_SIZE_BUTTON)));
-
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  m_notebook->signal_switch_page().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_switch_page));
-#endif /* !GLOM_ENABLE_CLIENT_ONLY */
-    
-  Glib::RefPtr<Gtk::TreeView::Selection> existing_view_selection = m_existing_view->get_selection();
-  existing_view_selection->signal_changed().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_existing_selection_changed));
-  existing_view_selection->set_select_function( sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_existing_select_func) ); 
-
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  builder->get_widget("existing_or_new_new_treeview", m_new_view);
-  g_assert(m_new_view);
-  m_new_model = Gtk::TreeStore::create(m_new_columns);
-  m_new_view->set_model(m_new_model);
-  
-  m_iter_new_empty = m_new_model->append();
-  (*m_iter_new_empty)[m_new_columns.m_col_title] = _("New Empty Document");
-
-  m_iter_new_template = m_new_model->append();
-  (*m_iter_new_template)[m_new_columns.m_col_title] = _("New From Template");
-  
-  m_new_column_title.set_expand(true);
-  m_new_column_title.pack_start(m_new_icon_renderer, false);
-  m_new_column_title.pack_start(m_new_title_renderer, true);
-  m_new_column_title.set_cell_data_func(m_new_icon_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::new_icon_data_func));
-  m_new_column_title.set_cell_data_func(m_new_title_renderer, sigc::mem_fun(*this, &Dialog_ExistingOrNew::new_title_data_func));
-  m_new_view->append_column(m_new_column_title);
-  
-  m_new_view->set_headers_visible(false);
-  m_new_view->signal_row_activated().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_row_activated));
-
-  m_iter_new_template_dummy = create_dummy_item_new(m_iter_new_template, _(TEMPLATE_DUMMY_TEXT));
- 
-  Glib::RefPtr<Gtk::TreeView::Selection> new_view_selection = m_new_view->get_selection();
-  new_view_selection->signal_changed().connect(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_selection_changed));
-  new_view_selection->set_select_function( sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_select_func) );
-#else /* GLOM_ENABLE_CLIENT_ONLY */
-  m_notebook->remove_page(NEW_PAGE);
-  m_notebook->set_show_tabs(false);
-#endif /* !GLOM_ENABLE_CLIENT_ONLY */
-    
-  //Make sure the first item is visible, 
-  //which is not always the case on Maemo. 
-  m_existing_view->scroll_to_row( 
+  //Make sure the first item is visible,
+  //which is not always the case on Maemo.
+  m_existing_view->scroll_to_row(
     Gtk::TreeModel::Path(m_iter_existing_other) );
 
   update_ui_sensitivity();
@@ -273,22 +271,34 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 bool Dialog_ExistingOrNew::list_examples_at_path(const std::string& path)
 {
-  //std::cout << "Debug: Dialog_ExistingOrNew::list_examples_at_path(): path=" << path << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": path=" << path << std::endl;
 
-  m_examples_dir = Gio::File::create_for_path(path);
+  Glib::RefPtr<Gio::File> examples_dir = Gio::File::create_for_path(path);
+  Glib::RefPtr<Gio::FileInfo> info;
 
   try
   {
-    m_examples_dir->enumerate_children_async(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_enumerate_children),
-                                             G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE","G_FILE_ATTRIBUTE_STANDARD_NAME);
+    Glib::RefPtr<Gio::FileEnumerator> examples = examples_dir->enumerate_children(G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE","G_FILE_ATTRIBUTE_STANDARD_NAME);
+    bool example_found = false;
+    while(info = examples->next_file())
+    {
+      const Glib::ustring title = get_title_from_example(info, examples_dir);
+      if(!title.empty())
+      {
+        append_example(title, Gio::File::create_for_path(Glib::build_filename(examples_dir->get_path(), info->get_name())));
+      	example_found = true;
+      }
+    }
+
     // TODO: Monitor example directory for new/removed files?
-    return true;
+    return example_found;
   }
   catch(const Glib::Exception& ex)
   {
-    std::cerr << "Could not enumerate examples at path=:" << path << ex.what() << std::endl;
-    return false;
+    std::cerr << "Could not enumerate examples at path=" << path << ", Error=" << ex.what() << std::endl;
   }
+
+  return false;
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
@@ -370,7 +380,7 @@ Dialog_ExistingOrNew::Action Dialog_ExistingOrNew::get_action_impl(Gtk::TreeMode
       return NONE;
     #endif //GLOM_ENABLE_CLIENT_ONLY
   }
-  
+
   return NONE;
 }
 
@@ -390,7 +400,7 @@ Glib::ustring Dialog_ExistingOrNew::get_uri() const
   {
     return (*iter)[m_new_columns.m_col_template_uri];
   }
-  else 
+  else
   #endif //GLOM_ENABLE_CLIENT_ONLY
   if(action == OPEN_URI)
   {
@@ -406,9 +416,7 @@ Glib::ustring Dialog_ExistingOrNew::get_uri() const
   }
   else
   {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     throw std::logic_error("Dialog_ExistingOrNew::get_uri: action is neither NEW_FROM_TEMPLATE nor OPEN_URI");
-#endif    
   }
 
   return Glib::ustring();
@@ -422,10 +430,8 @@ EpcServiceInfo* Dialog_ExistingOrNew::get_service_info() const
 
   if(action == OPEN_REMOTE)
     return (*iter)[m_existing_columns.m_col_service_info];
-#ifdef GLIBMM_EXCEPTIONS_ENABLED    
   else
     throw std::logic_error("Dialog_ExistingOrNew::get_service_info: action is not OPEN_REMOTE");
-#endif
 
   return 0;
 }
@@ -437,10 +443,8 @@ Glib::ustring Dialog_ExistingOrNew::get_service_name() const
 
   if(action == OPEN_REMOTE)
     return (*iter)[m_existing_columns.m_col_service_name];
-#ifdef GLIBMM_EXCEPTIONS_ENABLED    
   else
     throw std::logic_error("Dialog_ExistingOrNew::get_service_name: action is not OPEN_REMOTE");
-#endif
 
   return Glib::ustring();
 }
@@ -464,35 +468,16 @@ std::auto_ptr<Gtk::TreeModel::iterator> Dialog_ExistingOrNew::create_dummy_item_
 
 void Dialog_ExistingOrNew::existing_icon_data_func(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter)
 {
-  Gtk::CellRendererPixbuf* pixbuf_renderer = dynamic_cast<Gtk::CellRendererPixbuf*>(renderer);  
-  if(!pixbuf_renderer) 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  Gtk::CellRendererPixbuf* pixbuf_renderer = dynamic_cast<Gtk::CellRendererPixbuf*>(renderer);
+  if(!pixbuf_renderer)
   throw std::logic_error("Renderer not a pixbuf renderer in existing_icon_data_func");
-#else
-  return;
-#endif
 
-#ifdef GLIBMM_PROPERTIES_ENABLED
   pixbuf_renderer->property_stock_size() = Gtk::ICON_SIZE_BUTTON;
   pixbuf_renderer->property_stock_id() = "";
   pixbuf_renderer->property_pixbuf() = Glib::RefPtr<Gdk::Pixbuf>();
 
   if(iter == m_iter_existing_recent)
     pixbuf_renderer->property_stock_id() = Gtk::Stock::INDEX.id; // TODO: More meaningful icon?
-#ifndef G_OS_WIN32
-  else if(iter == m_iter_existing_network)
-    pixbuf_renderer->property_stock_id() = Gtk::Stock::NETWORK.id;
-#endif
-  else if(iter == m_iter_existing_other)
-    pixbuf_renderer->property_stock_id() = Gtk::Stock::OPEN.id;
-  else if(m_iter_existing_recent_dummy.get() && iter == *m_iter_existing_recent_dummy)
-    pixbuf_renderer->property_stock_id() = ""; // TODO: Use Stock::STOP instead?
-#ifndef G_OS_WIN32
-  else if(m_iter_existing_network_dummy.get() && iter == *m_iter_existing_network_dummy)
-    pixbuf_renderer->property_stock_id() = ""; // TODO: Use Stock::STOP instead?
-#endif
-
-#else // Properties enabled
 
   pixbuf_renderer->set_property("stock-size", Gtk::ICON_SIZE_BUTTON);
   pixbuf_renderer->set_property("stock-id", std::string());
@@ -512,8 +497,6 @@ void Dialog_ExistingOrNew::existing_icon_data_func(Gtk::CellRenderer* renderer,
   else if(m_iter_existing_network_dummy.get() && iter == *m_iter_existing_network_dummy)
     pixbuf_renderer->set_property("stock-id", std::string()); // TODO: Use Stock::STOP instead?
 #endif
-
-#endif // Properties enabled
   else
   {
     if(m_existing_model->is_ancestor(m_iter_existing_recent, iter))
@@ -531,9 +514,7 @@ void Dialog_ExistingOrNew::existing_icon_data_func(Gtk::CellRenderer* renderer,
 #endif
     else
     {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
       throw std::logic_error("Unexpected iterator in existing_icon_data_func");
-#endif      
     }
   }
 }
@@ -542,18 +523,13 @@ void Dialog_ExistingOrNew::existing_title_data_func(Gtk::CellRenderer* renderer,
 {
   Gtk::CellRendererText* text_renderer = dynamic_cast<Gtk::CellRendererText*>(renderer);
   if(!text_renderer)
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
- throw std::logic_error("Renderer not a text renderer in existing_title_data_func");
-#else
-  return;
-#endif    
+    throw std::logic_error("Renderer not a text renderer in existing_title_data_func");
 
-#ifdef GLIBMM_PROPERTIES_ENABLED
   text_renderer->property_text() = (*iter)[m_existing_columns.m_col_title];
 
   // Default: Use default color:
   text_renderer->property_foreground_set() = false;
-  
+
   // Use grey if parent item has no children:
 #ifndef G_OS_WIN32
   if( (iter == m_iter_existing_network && m_iter_existing_network_dummy.get()) ||
@@ -564,22 +540,6 @@ void Dialog_ExistingOrNew::existing_title_data_func(Gtk::CellRenderer* renderer,
   {
     text_renderer->property_foreground() = "grey";
   }
-#else // PROPERTIES_ENABLED
-  Glib::ustring text = (*iter)[m_existing_columns.m_col_title];
-  text_renderer->set_property("text", text);
-  // Default: Use default color
-  text_renderer->set_property("foreground-set", false);
-  // Use grey if parent item has no children
-#ifndef G_OS_WIN32
-  if( (iter == m_iter_existing_network && m_iter_existing_network_dummy.get()) ||
-      (iter == m_iter_existing_recent && m_iter_existing_recent_dummy.get()))
-#else
-  if(iter == m_iter_existing_recent && m_iter_existing_recent_dummy.get())
-#endif
-  {
-    text_renderer->set_property("foreground", std::string("grey"));
-  } 
-#endif // PROPERTIES_ENABLED
 }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -587,17 +547,12 @@ void Dialog_ExistingOrNew::new_icon_data_func(Gtk::CellRenderer* renderer, const
 {
   Gtk::CellRendererPixbuf* pixbuf_renderer = dynamic_cast<Gtk::CellRendererPixbuf*>(renderer);
   if(!pixbuf_renderer)
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  throw std::logic_error("Renderer not a pixbuf renderer in new_icon_data_func");
-#else
-  return;
-#endif
+    throw std::logic_error("Renderer not a pixbuf renderer in new_icon_data_func");
 
-#ifdef GLIBMM_PROPERTIES_ENABLED
   pixbuf_renderer->property_stock_size() = Gtk::ICON_SIZE_BUTTON;
   pixbuf_renderer->property_stock_id() = "";
   pixbuf_renderer->property_pixbuf() = Glib::RefPtr<Gdk::Pixbuf>();
-      
+
   if(iter == m_iter_new_empty)
     pixbuf_renderer->property_stock_id() = Gtk::Stock::NEW.id;
   else if(iter == m_iter_new_template)
@@ -612,49 +567,17 @@ void Dialog_ExistingOrNew::new_icon_data_func(Gtk::CellRenderer* renderer, const
     }
     else
     {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED    
       throw std::logic_error("Unexpected iterator in new_icon_data_func");
-#endif      
     }
   }
-#else
-  pixbuf_renderer->set_property("stock-size", Gtk::ICON_SIZE_BUTTON);
-  pixbuf_renderer->set_property("stock-id", Glib::ustring());
-  pixbuf_renderer->set_property("pixbuf", Glib::RefPtr<Gdk::Pixbuf>());
-      
-  if(iter == m_iter_new_empty)
-    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::NEW));
-  else if(iter == m_iter_new_template)
-    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::EDIT)); // TODO: More meaningful icon?
-  else if(m_iter_new_template_dummy.get() && iter == *m_iter_new_template_dummy)
-    pixbuf_renderer->set_property("stock-id", Gtk::StockID(Gtk::Stock::DIALOG_ERROR)); // TODO: Use Stock::STOP instead?
-  else
-  {
-    if(m_new_model->is_ancestor(m_iter_new_template, iter))
-    {
-      pixbuf_renderer->set_property("icon-name", Glib::ustring("glom"));
-    }
-    else
-    {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED    
-      throw std::logic_error("Unexpected iterator in new_icon_data_func");
-#endif      
-    }
-  }
-#endif 
 }
 
 void Dialog_ExistingOrNew::new_title_data_func(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter)
 {
   Gtk::CellRendererText* text_renderer = dynamic_cast<Gtk::CellRendererText*>(renderer);
-  if(!text_renderer) 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  if(!text_renderer)
     throw std::logic_error("Renderer not a text renderer in new_title_data_func");
-#else
-    return;
-#endif    
 
-#ifdef GLIBMM_PROPERTIES_ENABLED
   text_renderer->property_text() = (*iter)[m_new_columns.m_col_title];
 
   // Default: Use default color
@@ -664,22 +587,10 @@ void Dialog_ExistingOrNew::new_title_data_func(Gtk::CellRenderer* renderer, cons
   {
     text_renderer->property_foreground() = "grey";
   }
-#else
-  Glib::ustring text = (*iter)[m_new_columns.m_col_title];
-  text_renderer->set_property("text", text);
-
-  // Default: Use default color
-  text_renderer->set_property("foreground-set", false);
-  // Use grey if parent item has no children
-  if( (iter == m_iter_new_template && m_iter_new_template_dummy.get()))
-  {
-    text_renderer->set_property("foreground", Glib::ustring("grey"));
-  }
-#endif    
 }
 #endif //GLOM_ENABLE_CLIENT_ONLY
 
-void Dialog_ExistingOrNew::on_switch_page(GtkNotebookPage* /* page */, guint /* page_num */)
+void Dialog_ExistingOrNew::on_switch_page(Gtk::Widget* /* page */, guint /* page_num */)
 {
   update_ui_sensitivity();
 }
@@ -722,7 +633,7 @@ void Dialog_ExistingOrNew::update_ui_sensitivity()
 #endif
     }
   }
-  #ifndef GLOM_ENABLE_CLIENT_ONLY
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   else
   {
     const int count = m_new_view->get_selection()->count_selected_rows();
@@ -738,134 +649,81 @@ void Dialog_ExistingOrNew::update_ui_sensitivity()
                      (!m_iter_new_template_dummy.get() || sel != *m_iter_new_template_dummy));
     }
   }
-  #endif //GLOM_ENABLE_CLIENT_ONLY
+#endif //GLOM_ENABLE_CLIENT_ONLY
 
   m_select_button->set_sensitive(sensitivity);
 }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-void Dialog_ExistingOrNew::on_enumerate_children(const Glib::RefPtr<Gio::AsyncResult>& res)
-{
-  if(!m_examples_dir)
-    return;
-
-  try
-  {
-    m_examples_enumerator = m_examples_dir->enumerate_children_finish(res);
-    m_examples_enumerator->next_files_async(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_next_files));
-  }
-  catch(const Glib::Exception& ex)
-  {
-    std::cerr << "Could not enumerate files in examples directory: " << ex.what() << std::endl;
-    
-    m_examples_enumerator.reset();
-    m_examples_dir.reset();
-  }
-}
-
-void Dialog_ExistingOrNew::on_next_files(const Glib::RefPtr<Gio::AsyncResult>& res)
+Glib::ustring Dialog_ExistingOrNew::get_title_from_example(const Glib::RefPtr<Gio::FileInfo>& info, const Glib::RefPtr<Gio::File>& examples_dir)
 {
   try
   {
-    const Glib::ListHandle<Glib::RefPtr<Gio::FileInfo> >& list = m_examples_enumerator->next_files_finish(res);
-    if(list.empty())
+    // Load file.
+    const Glib::ustring content_type = info->get_attribute_string(G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE);
+    const Glib::ustring mime_type = Gio::content_type_get_mime_type(content_type);
+
+    if(mime_type == "application/x-glom")
     {
-      // Done
-      m_examples_dir.reset();
-      m_examples_enumerator.reset();
+      const Glib::RefPtr<Gio::File> current_example =
+        Gio::File::create_for_path(Glib::build_filename(examples_dir->get_path(), info->get_name()));
+      Glib::RefPtr<Gio::FileInputStream> stream = current_example->read();
+      m_current_buffer.reset(new buffer);
+      const int bytes_read = stream->read(m_current_buffer->buf, buffer::SIZE);
+      const std::string data(m_current_buffer->buf, bytes_read);
+      // TODO: Check that data is valid UTF-8, the last character might be truncated
+
+      Parser parser;
+      return (Glib::ustring(parser.get_example_title(data)));
     }
-    else
-    {
-      // Load file
-      Glib::RefPtr<Gio::FileInfo> info = *list.begin();
-      Glib::ustring content_type = info->get_attribute_string(G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE);
-      Glib::ustring mime_type = Gio::content_type_get_mime_type(content_type);
-
-      if(mime_type == "application/x-glom")
-      {
-        m_current_example = Gio::File::create_for_path(Glib::build_filename(m_examples_dir->get_path(), info->get_name()));
-        m_current_example->read_async(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_read));
-      }
-      else
-      {
-        // File is not a glom file, continue with next
-        m_examples_enumerator->next_files_async(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_next_files));
-      }
-    }
-  }
-  catch(const Glib::Exception& ex)
-  {
-    std::cerr << "Could not enumerate examples: " << ex.what() << std::endl;
-
-    m_examples_dir.reset();
-    m_examples_enumerator.reset();
-  }
-}
-
-void Dialog_ExistingOrNew::on_read(const Glib::RefPtr<Gio::AsyncResult>& res)
-{
-  try
-  {
-    m_current_stream = m_current_example->read_finish(res);
-    m_current_buffer.reset(new buffer);
-    m_current_stream->read_async(m_current_buffer->buf, buffer::SIZE, sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_stream_read));
   }
   catch(const Glib::Exception& exception)
   {
-    // Could not read this file, read next
-    m_examples_enumerator->next_files_async(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_next_files));
+    std::cerr << "Could not enumerate files in examples directory: " << exception.what() << std::endl;
 
-    m_current_example.reset();
-    m_current_stream.reset();
     m_current_buffer.reset();
   }
+
+  // File is not a glom file, continue with next.
+  return Glib::ustring();
 }
 
-void Dialog_ExistingOrNew::on_stream_read(const Glib::RefPtr<Gio::AsyncResult>& res)
+void Dialog_ExistingOrNew::append_example(const Glib::ustring& title, const Glib::RefPtr<Gio::File>& file)
 {
+  if(!m_new_model)
+  {
+    std::cerr << G_STRFUNC << ": m_new_model is null" << std::endl;
+    return;
+  }
+
+  if(!m_iter_new_template)
+  {
+    std::cerr << G_STRFUNC << ": m_iter_new_template is null" << std::endl;
+    return;
+  }
+
   try
   {
-    gssize size_read = m_current_stream->read_finish(res);
-    std::string data(m_current_buffer->buf, size_read);
-    // TODO: Check that data is valid UTF-8, the last character might be truncated
+    const bool is_first_item = m_iter_new_template_dummy.get();
 
-    Parser parser;
-    Glib::ustring title = parser.get_example_title(data);
+    // Add to list.
+    Gtk::TreeModel::iterator iter = m_new_model->append(m_iter_new_template->children());
+    (*iter)[m_new_columns.m_col_title] = title;
+    (*iter)[m_new_columns.m_col_template_uri] = file->get_uri();
 
-    if(title.empty())
+    if(is_first_item)
     {
-      // TODO: Read more data from this file?
-    }
-    else
-    {
-      const bool is_first_item = m_iter_new_template_dummy.get();
-
-      // Add to list
-      Gtk::TreeModel::iterator iter = m_new_model->append(m_iter_new_template->children());
-      (*iter)[m_new_columns.m_col_title] = title;
-      (*iter)[m_new_columns.m_col_template_uri] = m_current_example->get_uri();
-
-      if(is_first_item)
-      {
-        // Remove dummy
-        m_new_model->erase(*m_iter_new_template_dummy);
-        m_iter_new_template_dummy.reset();
-        // Expand if this is the first item
-        m_new_view->expand_row(m_new_model->get_path(m_iter_new_template), false);
-      }
+      // Remove dummy.
+      m_new_model->erase(*m_iter_new_template_dummy);
+      m_iter_new_template_dummy.reset();
+      // Expand if this is the first item.
+      m_new_view->expand_row(m_new_model->get_path(m_iter_new_template), false);
     }
   }
   catch(const Glib::Exception& ex)
   {
-    std::cerr << "Could not read " << m_current_example->get_path() << ": " << ex.what() << std::endl;
+    std::cerr << "Could not read example: " << file->get_path() << ": " << ex.what() << std::endl;
   }
-
-  // Done with this, read next file
-  m_current_example.reset();
-  m_current_stream.reset();
-  m_current_buffer.reset();
-
-  m_examples_enumerator->next_files_async(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_next_files));
 }
 #endif /* !GLOM_ENABLE_CLIENT_ONLY */
 
@@ -892,7 +750,7 @@ void Dialog_ExistingOrNew::on_service_found(const Glib::ustring& name, EpcServic
 
   epc_service_info_ref(info);
   g_free(title);
-  
+
   // Remove dummy item
   if(m_iter_existing_network_dummy.get())
   {
@@ -969,9 +827,9 @@ void Dialog_ExistingOrNew::on_select_clicked()
     dialog.set_default_response(Gtk::RESPONSE_OK);
     #endif // GLOM_ENABLE_MAEMO
 
-    Gtk::FileFilter filter;
-    filter.add_mime_type("application/x-glom");
-    filter.set_name("Glom files");
+    Glib::RefPtr<Gtk::FileFilter> filter = Gtk::FileFilter::create();
+    filter->add_mime_type("application/x-glom");
+    filter->set_name("Glom files");
     dialog.add_filter(filter);
 
     const int response_id = dialog.run();
