@@ -67,8 +67,6 @@
 #endif
 
 #include <glom/filechooser_export.h>
-#include <gtkmm/stock.h>
-#include <gtkmm/radioaction.h>
 #include <libglom/privs.h>
 #include <libglom/db_utils.h>
 #include <sstream> //For stringstream.
@@ -498,7 +496,7 @@ void Frame_Glom::show_table_allow_empty(const Glib::ustring& table_name, const G
     }
     default:
     {
-      std::cout << "Frame_Glom::on_box_tables_selected(): Unexpected mode" << std::endl;
+      std::cout << "debug: " << G_STRFUNC << ": Unexpected mode" << std::endl;
       strMode = _("Unknown");
       break;
     }
@@ -681,11 +679,11 @@ void Frame_Glom::on_menu_file_export()
 //TODO: Reduce copy/pasting in these export_data_to_*() methods:
 void Frame_Glom::export_data_to_vector(Document::type_example_rows& the_vector, const FoundSet& found_set, const Document::type_list_layout_groups& sequence)
 {
-  type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(found_set.m_table_name, sequence);
+  type_vecConstLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(found_set.m_table_name, sequence);
 
   if(fieldsSequence.empty())
   {
-    std::cerr << "Glom: Frame_Glom::export_data_to_string(): No fields in sequence." << std::endl;
+    std::cerr << G_STRFUNC << ": No fields in sequence." << std::endl;
     return;
   }
 
@@ -710,7 +708,7 @@ void Frame_Glom::export_data_to_vector(Document::type_example_rows& the_vector,
         {
           const Gnome::Gda::Value value = result->get_value_at(col_index, row_index);
 
-          sharedptr<LayoutItem_Field> layout_item = fieldsSequence[col_index];
+          sharedptr<const LayoutItem_Field> layout_item = fieldsSequence[col_index];
           //if(layout_item->m_field.get_glom_type() != Field::TYPE_IMAGE) //This is too much data.
           //{
 
@@ -731,11 +729,11 @@ void Frame_Glom::export_data_to_vector(Document::type_example_rows& the_vector,
 
 void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const FoundSet& found_set, const Document::type_list_layout_groups& sequence)
 {
-  type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(found_set.m_table_name, sequence);
+  type_vecConstLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(found_set.m_table_name, sequence);
 
   if(fieldsSequence.empty())
   {
-    std::cerr << "Glom: Frame_Glom::export_data_to_string(): No fields in sequence." << std::endl;
+    std::cerr << G_STRFUNC << ": No fields in sequence." << std::endl;
     return;
   }
 
@@ -760,7 +758,7 @@ void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const FoundSet
         {
           const Gnome::Gda::Value value = result->get_value_at(col_index, row_index);
 
-          sharedptr<LayoutItem_Field> layout_item = fieldsSequence[col_index];
+          sharedptr<const LayoutItem_Field> layout_item = fieldsSequence[col_index];
           //if(layout_item->m_field.get_glom_type() != Field::TYPE_IMAGE) //This is too much data.
           //{
             if(!row_string.empty())
@@ -783,11 +781,11 @@ void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const FoundSet
 
 void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet& found_set, const Document::type_list_layout_groups& sequence)
 {
-  type_vecLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(found_set.m_table_name, sequence);
+  type_vecConstLayoutFields fieldsSequence = get_table_fields_to_show_for_sequence(found_set.m_table_name, sequence);
 
   if(fieldsSequence.empty())
   {
-    std::cerr << "Glom: Frame_Glom::export_data_to_stream(): No fields in sequence." << std::endl;
+    std::cerr << G_STRFUNC << ": No fields in sequence." << std::endl;
     return;
   }
 
@@ -812,7 +810,7 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet&
         {
           const Gnome::Gda::Value value = result->get_value_at(col_index, row_index);
 
-          sharedptr<LayoutItem_Field> layout_item = fieldsSequence[col_index];
+          sharedptr<const LayoutItem_Field> layout_item = fieldsSequence[col_index];
           //if(layout_item->m_field.get_glom_type() != Field::TYPE_IMAGE) //This is too much data.
           //{
             if(!row_string.empty())
@@ -822,7 +820,7 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet&
             sharedptr<const Field> field = layout_item->get_full_field_details();
             if(!field)
             {
-              std::cerr << "Glom: Frame_Glom::export_data_to_stream(): A field was null." << std::endl;
+              std::cerr << G_STRFUNC << ": A field was null." << std::endl;
               return;
             }
 
@@ -880,13 +878,13 @@ void Frame_Glom::on_menu_file_import()
     Gtk::FileChooserDialog file_chooser(*get_app_window(), _("Open CSV Document"), Gtk::FILE_CHOOSER_ACTION_OPEN);
     file_chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
     file_chooser.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
-    Gtk::FileFilter filter_csv;
-    filter_csv.set_name(_("CSV files"));
-    filter_csv.add_mime_type("text/csv");
+    Glib::RefPtr<Gtk::FileFilter> filter_csv = Gtk::FileFilter::create();
+    filter_csv->set_name(_("CSV files"));
+    filter_csv->add_mime_type("text/csv");
     file_chooser.add_filter(filter_csv);
-    Gtk::FileFilter filter_any;
-    filter_any.set_name(_("All files"));
-    filter_any.add_pattern("*");
+    Glib::RefPtr<Gtk::FileFilter> filter_any = Gtk::FileFilter::create();
+    filter_any->set_name(_("All files"));
+    filter_any->add_pattern("*");
     file_chooser.add_filter(filter_any);
 
     if(file_chooser.run() == Gtk::RESPONSE_ACCEPT)
@@ -895,6 +893,9 @@ void Frame_Glom::on_menu_file_import()
 
       Dialog_Import_CSV* dialog = 0;
       Glom::Utils::get_glade_widget_derived_with_warning(dialog);
+      if(!dialog) //Unlikely and it already warns on stderr.
+        return;
+
       add_view(dialog);
 
       dialog->import(file_chooser.get_uri(), m_table_name);
@@ -933,7 +934,7 @@ void Frame_Glom::on_menu_file_toggle_share(const Glib::RefPtr<Gtk::ToggleAction>
 {
   if(!action)
   {
-    std::cerr << "Frame_Glom::on_menu_file_toggle_share(): action was null." << std::endl;
+    std::cerr << G_STRFUNC << ": action was null." << std::endl;
   }
 
   //Prevent this change if not in developer mode,
@@ -1040,7 +1041,7 @@ void Frame_Glom::on_menu_file_toggle_share(const Glib::RefPtr<Gtk::ToggleAction>
 
           if(!reowned || !(removed || disabled))
           {
-            std::cerr << "Frame_Glom::on_menu_file_toggle_share(): Failed to reown and remove/revoke default user." << std::endl;
+            std::cerr << G_STRFUNC << ": Failed to reown and remove/revoke default user." << std::endl;
             shared = false;
             change = false;
           }
@@ -1315,7 +1316,7 @@ void Frame_Glom::on_dialog_add_related_table_response(int response)
       const bool result = DbUtils::create_table_with_default_fields(get_document(), table_name);
       if(!result)
       {
-        std::cerr << "Frame_Glom::on_menu_Tables_AddRelatedTable(): create_table_with_default_fields() failed." << std::endl;
+        std::cerr << G_STRFUNC << ": create_table_with_default_fields() failed." << std::endl;
         return;
       }
 
@@ -1331,7 +1332,7 @@ void Frame_Glom::on_dialog_add_related_table_response(int response)
       sharedptr<Field> related_primary_key = get_field_primary_key_for_table(table_name); //This field was created by create_table_with_default_fields().
       if(!related_primary_key)
       {
-        std::cerr << "Frame_Glom::on_menu_Tables_AddRelatedTable(): get_field_primary_key_for_table() failed." << std::endl;
+        std::cerr << G_STRFUNC << ": get_field_primary_key_for_table() failed." << std::endl;
         return;
       }
 
@@ -1473,15 +1474,14 @@ void Frame_Glom::on_button_quickfind()
   else
   {
     const Gnome::Gda::SqlExpr where_clause = Utils::get_find_where_clause_quick(get_document(), m_table_name, Gnome::Gda::Value(criteria));
-
-    //std::cout << "debug: " << G_STRFUNC << ": where_clause=" << where_clause.serialize() << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": where_clause=" << where_clause << std::endl;
     on_notebook_find_criteria(where_clause);
   }
 }
 
 void Frame_Glom::on_notebook_find_criteria(const Gnome::Gda::SqlExpr& where_clause)
 {
-  //std::cout << "Frame_Glom::on_notebook_find_criteria(): " << where_clause << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": " << where_clause << std::endl;
 
   Application* pApp = dynamic_cast<Application*>(get_app_window());
   if(pApp)
@@ -1499,9 +1499,9 @@ void Frame_Glom::on_notebook_find_criteria(const Gnome::Gda::SqlExpr& where_clau
       found_set.m_where_clause = where_clause;
       records_found = m_Notebook_Data.init_db_details(found_set);
 
-      //std::cout << "Frame_Glom::on_notebook_find_criteria(): BEFORE  m_Notebook_Data.select_page_for_find_results()" << std::endl;
+      //std::cout << "debug: " << G_STRFUNC << ": BEFORE  m_Notebook_Data.select_page_for_find_results()" << std::endl;
       m_Notebook_Data.select_page_for_find_results();
-      //std::cout << "Frame_Glom::on_notebook_find_criteria(): AFTER  m_Notebook_Data.select_page_for_find_results()" << std::endl;
+      //std::cout << "debug: " << G_STRFUNC << ": AFTER  m_Notebook_Data.select_page_for_find_results()" << std::endl;
     }
 
     if(!records_found)
@@ -1686,6 +1686,9 @@ void Frame_Glom::on_menu_developer_database_preferences()
 {
   Dialog_Database_Preferences* dialog = 0;
   Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return;
+        
   dialog->set_transient_for(*(get_app_window()));
   add_view(dialog);
   dialog->load_from_document();
@@ -1718,9 +1721,7 @@ void Frame_Glom::do_menu_developer_fields(Gtk::Window& parent, const Glib::ustri
   // Some database backends (SQLite) require the table to change to no longer
   // be in use when changing the records, so we stop the database usage
   // here. We reshow everything in on_developer_dialog_hide() anyway.
-  Document* document = dynamic_cast<Document*>(get_document());
-  if(document && document->get_hosting_mode() == Document::HOSTING_MODE_SQLITE)
-    show_no_table();
+  show_no_table();
 
   // Remember the old table name so that we re-show the previous table as
   // soon as the dialog has been closed.
@@ -1784,13 +1785,18 @@ void Frame_Glom::on_menu_developer_relationships()
   if(m_table_name.empty())
     return;
 
-  do_menu_developer_relationships(*get_app_window(), m_table_name);
+  Gtk::Window* app = get_app_window();
+  if(app)
+    do_menu_developer_relationships(*app, m_table_name);
 }
 
 void Frame_Glom::on_menu_developer_users()
 {
   Dialog_GroupsList* dialog = 0;
   Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return;
+    
   dialog->set_transient_for(*get_app_window());
 
   add_view(dialog); //Give it access to the document.
@@ -1876,6 +1882,9 @@ void Frame_Glom::on_menu_developer_script_library()
 {
   Dialog_ScriptLibrary* dialog = 0;
   Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return;
+    
   dialog->set_transient_for(*(get_app_window()));
   add_view(dialog); //Give it access to the document.
   dialog->load_from_document();
@@ -2161,13 +2170,13 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
   }
 
   // Do startup, such as starting the self-hosting database server
-  const ConnectionPool::StartupErrors started = 
+  const ConnectionPool::StartupErrors started =
     connection_pool->startup( sigc::mem_fun(*this, &Frame_Glom::on_connection_startup_progress) );
   if(started != ConnectionPool::Backend::STARTUPERROR_NONE)
   {
     std::cerr << G_STRFUNC << ": startup() failed." << std::endl;
     return false;
-  }  
+  }
 
   if(m_dialog_progess_connection_startup)
   {
@@ -2206,7 +2215,6 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
     m_pDialogConnection->set_database_name(database_name_possible);
     //std::cout << "debug: possible name=" << database_name_possible << std::endl;
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
       g_assert(m_pDialogConnection);
@@ -2216,14 +2224,6 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
     }
     catch(const ExceptionConnection& ex)
     {
-#else //GLIBMM_EXCEPTIONS_ENABLED
-    std::auto_ptr<ExceptionConnection> error;
-    g_assert(m_pDialogConnection);
-    sharedptr<SharedConnection> sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings(error);
-    if(error.get())
-    {
-      const ExceptionConnection& ex = *error;
-#endif //GLIBMM_EXCEPTIONS_ENABLED
       //g_warning("Frame_Glom::connection_request_password_and_choose_new_database_name(): caught exception.");
 
       if(ex.get_failure_type() == ExceptionConnection::FAILURE_NO_SERVER)
@@ -2234,7 +2234,7 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
       }
       else
       {
-        std::cout << "Frame_Glom::connection_request_password_and_choose_new_database_name(): unused database name successfully found: " << database_name_possible << std::endl;
+        std::cout << "debug: " << G_STRFUNC << ": unused database name successfully found: " << database_name_possible << std::endl;
         //The connection to the server is OK, but the specified database does not exist.
         //That's good - we were looking for an unused database name.
 
@@ -2319,7 +2319,7 @@ bool Frame_Glom::handle_request_password_connection_error(bool asked_for_passwor
   }
   else
   {
-    std::cerr << "Frame_Glom::connection_request_password_and_attempt(): Unexpected exception: " << ex.what() << std::endl;
+    std::cerr << G_STRFUNC << ": Unexpected exception: " << ex.what() << std::endl;
     cleanup_connection();
     return false;
   }
@@ -2338,7 +2338,7 @@ bool Frame_Glom::connection_request_password_and_attempt(bool& database_not_foun
   //Start a self-hosted server if necessary:
   ConnectionPool* connection_pool = ConnectionPool::get_instance();
   connection_pool->setup_from_document(document);
-  const ConnectionPool::StartupErrors started = 
+  const ConnectionPool::StartupErrors started =
     connection_pool->startup( sigc::mem_fun(*this, &Frame_Glom::on_connection_startup_progress) );
   if(started != ConnectionPool::Backend::STARTUPERROR_NONE)
   {
@@ -2409,7 +2409,6 @@ bool Frame_Glom::connection_request_password_and_attempt(bool& database_not_foun
       //TODO: Remove any previous database setting?
       if(m_pDialogConnection)
       {
-        #ifdef GLIBMM_EXCEPTIONS_ENABLED
         try
         {
           sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings();
@@ -2421,15 +2420,6 @@ bool Frame_Glom::connection_request_password_and_attempt(bool& database_not_foun
           if(!handle_request_password_connection_error(true, ex, database_not_found))
             return false;
         }
-        #else //GLIBMM_EXCEPTIONS_ENABLED
-        std::auto_ptr<ExceptionConnection> local_error;
-        sharedconnection =
-          m_pDialogConnection->connect_to_server_with_connection_settings(local_error);
-        if(!local_error.get())
-          return true;
-        else if(!handle_request_password_connection_error(true, *local_error, database_not_found))
-          return false;
-        #endif //GLIBMM_EXCEPTIONS_ENABLED
       }
       else
       {
@@ -2438,7 +2428,6 @@ bool Frame_Glom::connection_request_password_and_attempt(bool& database_not_foun
         connectionpool->set_user(known_username);
         connectionpool->set_password(known_password);
 
-        #ifdef GLIBMM_EXCEPTIONS_ENABLED
         try
         {
           Base_DB::connect_to_server(get_app_window());
@@ -2449,17 +2438,6 @@ bool Frame_Glom::connection_request_password_and_attempt(bool& database_not_foun
           if(!handle_request_password_connection_error(false, ex, database_not_found))
             return false;
         }
-        #else
-        std::auto_ptr<ExceptionConnection> error;
-        const bool connected = Base_DB::connect_to_server(get_app_window(), error);
-        if(!connected || error.get())
-        {
-          if(!handle_request_password_connection_error(false, *error, database_not_found))
-            return false;
-        }
-        else
-          return true;
-        #endif
       }
 
       //Try again.
@@ -2680,7 +2658,7 @@ void Frame_Glom::on_dialog_tables_hide()
   }
 }
 
-void Frame_Glom::on_notebook_data_switch_page(GtkNotebookPage* /* page */, guint /* page_num */)
+void Frame_Glom::on_notebook_data_switch_page(Gtk::Widget* /* page */, guint /* page_num */)
 {
   //Refill this menu, because it depends on whether list or details are visible:
   Application* pApp = dynamic_cast<Application*>(get_app_window());
