@@ -18,8 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef FRAME_GLOM_H
-#define FRAME_GLOM_H
+#ifndef GLOM_FRAME_GLOM_H
+#define GLOM_FRAME_GLOM_H
 
 #include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
 
@@ -232,7 +232,7 @@ protected:
   void on_notebook_find_criteria(const Gnome::Gda::SqlExpr& where_clause);
   void on_button_quickfind();
   void on_button_find_all();
-  void on_notebook_data_switch_page(GtkNotebookPage* page, guint page_num);
+  void on_notebook_data_switch_page(Gtk::Widget* page, guint page_num);
   void on_notebook_data_record_details_requested(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value);
   void on_userlevel_changed(AppState::userlevels userlevel);
 
@@ -313,4 +313,4 @@ private:
 
 } //namespace Glom
 
-#endif //FRAME_GLOM_H
+#endif // GLOM_FRAME_GLOM_H
