@@ -19,8 +19,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef GLOM_IMPORT_CSV_H
-#define GLOM_IMPORT_CSV_H
+#ifndef GLOM_CSV_PARSER_H
+#define GLOM_CSV_PARSER_H
 
 #include "base_db.h"
 
@@ -48,7 +48,7 @@ namespace Glom
  *
  * set_file_and_start_parsing() to start parsing.
  * The data can then be read via get_date(), with get_rows_count() and get_cols_count().
- * 
+ *
  * The signals offer feedback while the parsing is happening.
  */
 class CsvParser
@@ -64,7 +64,7 @@ public:
   ~CsvParser();
 
   enum State {
-    STATE_NONE, 
+    STATE_NONE,
     STATE_PARSING,  /**< Parsing is in progress. */
     STATE_ENCODING_ERROR, /**< An error happened while parsing. */
     STATE_PARSED /**< Finished parsing. */
@@ -99,8 +99,8 @@ public:
 
   typedef sigc::signal<void, const Glib::ustring&> type_signal_have_display_name;
 
-  /** This signal will be emitted when the parser has discovered the 
-   * display name for the file. This does not require any parsing of the contents, 
+  /** This signal will be emitted when the parser has discovered the
+   * display name for the file. This does not require any parsing of the contents,
    * but it is asynchronous, so CsvParser signals this as a convenience.
    */
   type_signal_have_display_name signal_have_display_name() const;
@@ -139,8 +139,8 @@ public:
   void clear();
 
   /** Change the encoding used when reading the file.
-   * This stop parsing. Call set_file_and_start_parsing() to restart the parser  
-   * with the specified encoding. 
+   * This stop parsing. Call set_file_and_start_parsing() to restart the parser
+   * with the specified encoding.
    * See the FileEncoding namespace.
    */
   void set_encoding(const Glib::ustring& encoding_charset);
@@ -164,6 +164,8 @@ private:
   //TODO: Document this:
   static Glib::ustring::const_iterator advance_field(const Glib::ustring::const_iterator& iter, const Glib::ustring::const_iterator& end, Glib::ustring& field);
 
+  void ensure_idle_handler_connection();
+
   void on_file_read(const Glib::RefPtr<Gio::AsyncResult>& result, const Glib::RefPtr<Gio::File>& source);
   void copy_buffer_and_continue_reading(gssize size);
   void on_buffer_read(const Glib::RefPtr<Gio::AsyncResult>& result);
@@ -179,7 +181,6 @@ private:
   std::string m_encoding;
   std::vector<char>::size_type m_input_position;
   std::string m_current_line;
-  bool m_in_quotes;
 
   sigc::connection m_idle_connection;
   unsigned int m_line_number;
@@ -210,6 +211,4 @@ private:
 
 } //namespace Glom
 
-#endif //GLOM_IMPORT_CSV_H
-
-
+#endif //GLOM_CSV_PARSER_H
