@@ -27,9 +27,7 @@
 
 #include <libglom/connectionpool_backends/postgres_central.h>
 #include <libglom/connectionpool_backends/postgres_self.h>
-# include <libglom/connectionpool_backends/sqlite.h>
-
-#include <glibmm/main.h>
+#include <libglom/connectionpool_backends/sqlite.h>
 
 #ifdef G_OS_WIN32
 # include <windows.h>
@@ -40,8 +38,6 @@
 
 #include <signal.h> //To catch segfaults
 
-#include <glibmm/convert.h>
-#include <glibmm/miscutils.h>
 #include <glibmm/i18n.h>
 
 #include <iostream>
@@ -110,7 +106,8 @@ ConnectionPool::ConnectionPool()
   m_sharedconnection_refcount(0),
   m_ready_to_connect(false),
   m_pFieldTypes(0),
-  m_show_debug_output(false)
+  m_show_debug_output(false),
+  m_auto_server_shutdown(true)
 {
 }
 
@@ -166,7 +163,7 @@ void ConnectionPool::setup_from_document(const Document* document)
 
   default:
     //on_document_load() should have checked for this already, informing the user.
-    std::cerr << "Glom: setup_connection_pool_from_document(): Unhandled hosting mode: " << document->get_hosting_mode() << std::endl;
+    std::cerr << G_STRFUNC << ": Unhandled hosting mode: " << document->get_hosting_mode() << std::endl;
     g_assert_not_reached();
     break;
   }
@@ -212,20 +209,10 @@ const ConnectionPool::Backend* ConnectionPool::get_backend() const
   return m_backend.get();
 }
 
-bool ConnectionPool::get_backend_supports_cursor() const
-{
-  //TODO: Is there a generic libgda way to discover this?
-  const ConnectionPoolBackends::Sqlite* sqlite_backend =
-    dynamic_cast<const ConnectionPoolBackends::Sqlite*>(get_backend());
-  return !sqlite_backend;
-}
+
 
 //static:
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
 sharedptr<SharedConnection> ConnectionPool::get_and_connect()
-#else
-sharedptr<SharedConnection> ConnectionPool::get_and_connect(std::auto_ptr<ExceptionConnection>& error)
-#endif // GLIBMM_EXCEPTIONS_ENABLED
 {
   sharedptr<SharedConnection> result(0);
 
@@ -235,15 +222,11 @@ sharedptr<SharedConnection> ConnectionPool::get_and_connect(std::auto_ptr<Except
 
   if(!(connection_pool->m_backend.get()))
   {
-    std::cerr << "ConnectionPool::get_and_connect(): m_backend is null." << std::endl;
+    std::cerr << G_STRFUNC << ": m_backend is null." << std::endl;
     return result; //TODO: Return a FAILURE_NO_BACKEND error?, though that would be tedious.
   }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   result = connection_pool->connect();
-#else
-  result = connection_pool->connect(error);
-#endif // GLIBMM_EXCEPTIONS_ENABLED
 
   return result;
 }
@@ -269,11 +252,7 @@ static bool on_connection_pool_cache_timeout()
 }
 
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
 sharedptr<SharedConnection> ConnectionPool::connect()
-#else
-sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConnection>& error)
-#endif
 {
   //Don't try to connect if we don't have a backend to connect to.
   g_return_val_if_fail(m_backend.get(), sharedptr<SharedConnection>(0));
@@ -312,41 +291,31 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
     }
     else
     {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-      std::auto_ptr<ExceptionConnection> error;
-#endif
-      m_refGdaConnection = m_backend->connect(m_database, get_user(), get_password(), error);
-
-      if(!m_refGdaConnection)
+      try
       {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-        throw *error;
-#endif
-        return sharedptr<SharedConnection>(0);
+        m_refGdaConnection = m_backend->connect(m_database, get_user(), get_password());
       }
-      else
+      catch(const Glib::Error& ex)
+      {
+        throw ex;
+      }
+
       {
         //Allow get_meta_store_data() to succeed:
         //Hopefully this (and the update_meta_store_for_table() calls) is all we need.
         //std::cout << "DEBUG: Calling update_meta_store_data_types() ..." << std::endl;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
         try
         {
           m_refGdaConnection->update_meta_store_data_types();
         }
         catch(const Glib::Error& ex)
         {
-          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex.what() << std::endl;
+          std::cerr << G_STRFUNC << ": update_meta_store_data_types() failed: " << ex.what() << std::endl;
         }
         //std::cout << "DEBUG: ... update_meta_store_data_types() has finished." << std::endl;
-#else
-        std::auto_ptr<Glib::Error> ex;
-        m_refGdaConnection->update_meta_store_data_types(ex);
-        if (ex.get())
-          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex->what() << std::endl;
-#endif
+
         //std::cout << "DEBUG: Calling update_meta_store_table_names() ..." << std::endl;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
+
         try
         {
           //update_meta_store_table_names() has been known to throw an exception.
@@ -355,14 +324,9 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
         }
         catch(const Glib::Error& ex)
         {
-          std::cerr << "ConnectionPool::connect(): update_meta_store_table_names() failed: " << ex.what() << std::endl;
+          std::cerr << G_STRFUNC << ": update_meta_store_table_names() failed: " << ex.what() << std::endl;
         }
         //std::cout << "DEBUG: ... update_meta_store_table_names() has finished." << std::endl;
-#else
-        m_refGdaConnection->update_meta_store_table_names(m_backend->get_public_schema_name(), ex);
-        if (ex.get())
-          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex->what() << std::endl;
-#endif
 
         // Connection succeeded
         // Create the fieldtypes member if it has not already been done:
@@ -380,11 +344,7 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
           avahi_start_publishing(); //Stopped in the signal_finished handler.
 #endif // !G_OS_WIN32
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
         return connect(); //Call this method recursively. This time m_refGdaConnection exists.
-#else
-        return connect(error); //Call this method recursively. This time m_refGdaConnection exists.
-#endif // GLIBMM_EXCEPTIONS_ENABLED
       }
     }
   }
@@ -396,21 +356,10 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
   return sharedptr<SharedConnection>(0);
 }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
 void ConnectionPool::create_database(const Glib::ustring& database_name)
-#else
-void ConnectionPool::create_database(const Glib::ustring& database_name, std::auto_ptr<Glib::Error>& error)
-#endif
 {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  std::auto_ptr<Glib::Error> error;
-#endif
   if(m_backend.get())
-    m_backend->create_database(database_name, get_user(), get_password(), error);
-
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  if(error.get()) throw *error;
-#endif
+    m_backend->create_database(database_name, get_user(), get_password());
 }
 
 void ConnectionPool::set_user(const Glib::ustring& value)
@@ -418,7 +367,7 @@ void ConnectionPool::set_user(const Glib::ustring& value)
   if(value.empty())
   {
 #ifdef GLOM_CONNECTION_DEBUG
-    std::cout << "debug: ConnectionPool::set_user(): user is empty." << std::endl;
+    std::cout << "debug: " << G_STRFUNC << ": user is empty." << std::endl;
 #endif
   }
 
@@ -524,7 +473,7 @@ const FieldTypes* ConnectionPool::get_field_types() const
   return m_pFieldTypes;
 }
 
-Gnome::Gda::SqlOperatorType ConnectionPool::get_string_find_operator() const
+Glib::ustring ConnectionPool::get_string_find_operator() const
 {
   g_assert(m_backend.get());
   return m_backend->get_string_find_operator();
@@ -568,18 +517,13 @@ void ConnectionPool::on_sharedconnection_finished()
 //static
 bool ConnectionPool::handle_error_cerr_only()
 {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   sharedptr<SharedConnection> sharedconnection = get_and_connect();
-#else
-  std::auto_ptr<ExceptionConnection> conn_error;
-  sharedptr<SharedConnection> sharedconnection = get_and_connect(conn_error);
-  // Ignore error, sharedconnection presence is checked below
-#endif
+
   if(sharedconnection)
   {
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
 
-    typedef std::vector< Glib::RefPtr<Gnome::Gda::ConnectionEvent> > type_list_errors;
+    typedef std::list< Glib::RefPtr<Gnome::Gda::ConnectionEvent> > type_list_errors;
     type_list_errors list_errors = gda_connection->get_events();
 
     if(!list_errors.empty())
@@ -620,7 +564,7 @@ static void on_linux_signal(int signum)
 {
   ConnectionPool* connection_pool = ConnectionPool::get_instance();
   if(!connection_pool)
-  return;
+    return;
 
   if(signum == SIGSEGV)
   {
@@ -651,7 +595,8 @@ ConnectionPool::StartupErrors ConnectionPool::startup(const SlotProgress& slot_p
 
   //If we crash while running (unlikely, hopefully), then try to cleanup.
   //Comment this out if you want to see the backtrace in a debugger.
-  previous_sig_handler = signal(SIGSEGV, &on_linux_signal);
+  if(m_auto_server_shutdown)
+    previous_sig_handler = signal(SIGSEGV, &on_linux_signal);
 
   return started;
 }
@@ -682,8 +627,11 @@ bool ConnectionPool::cleanup(const SlotProgress& slot_progress)
 #endif // !G_OS_WIN32
 
   //We don't need the segfault handler anymore:
-  signal(SIGSEGV, previous_sig_handler);
-  previous_sig_handler = SIG_DFL; /* Arbitrary default */
+  if(previous_sig_handler != SIG_DFL) /* Arbitrary default */
+  {
+    signal(SIGSEGV, previous_sig_handler);
+    previous_sig_handler = SIG_DFL; /* Arbitrary default */
+  }
 
   return result;
 }
@@ -697,134 +645,78 @@ bool ConnectionPool::set_network_shared(const SlotProgress& slot_progress, bool
     return false;
 }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field)
-#else
-bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field, std::auto_ptr<Glib::Error>& error)
-#endif
+bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field) throw()
 {
   sharedptr<SharedConnection> conn;
   if(!m_refGdaConnection)
   {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     conn = connect();
-#else
-    std::auto_ptr<ExceptionConnection> local_error;
-    // TODO: We can't rethrow local_error here since ExceptionConnection does
-    // not derive from Glib::Error
-    conn = connect(local_error);
-#endif
-  }
-
-  if(!m_refGdaConnection)
-    return false;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  std::auto_ptr<Glib::Error> error;
-#endif
-  const bool result = m_backend->add_column(m_refGdaConnection, table_name, field, error);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  if(error.get()) throw *error;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
-#else
-  if(error.get())
-    std::cerr << "Error: " << error->what() << std::endl;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
-  if(error.get())
-    std::cerr << "Error: " << error->what() << std::endl;
-#endif
-  return result;
-}
-
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-bool ConnectionPool::drop_column(const Glib::ustring& table_name, const Glib::ustring& field_name)
-#else
-bool ConnectionPool::drop_column(const Glib::ustring& table_name, const Glib::ustring& field_name, std::auto_ptr<Glib::Error>& error)
-#endif
-{
-  sharedptr<SharedConnection> conn;
-  if(!m_refGdaConnection)
-  {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-    conn = connect();
-#else
-    std::auto_ptr<ExceptionConnection> local_error;
-    // TODO: We can't rethrow local_error here since ExceptionConnection does
-    // not derive from Glib::Error
-    conn = connect(local_error);
-#endif
   }
 
   if(!m_refGdaConnection)
     return false;
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  std::auto_ptr<Glib::Error> error;
-#endif
-  const bool result = m_backend->drop_column(m_refGdaConnection, table_name, field_name, error);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  if(error.get()) throw *error;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
-#else
-  if(error.get())
-    std::cerr << "Error: " << error->what() << std::endl;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
-  if(error.get())
-    std::cerr << "Error: " << error->what() << std::endl;
-#endif
-  return result;
+  try
+  {
+    m_backend->add_column(m_refGdaConnection, table_name, field);
+    m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
+    return true;
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception:" << ex.what() << std::endl;
+  }
+
+  return false;
 }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-bool ConnectionPool::change_column(const Glib::ustring& table_name, const sharedptr<const Field>& field_old, const sharedptr<const Field>& field)
-#else
-bool ConnectionPool::change_column(const Glib::ustring& table_name, const sharedptr<const Field>& field_old, const sharedptr<const Field>& field, std::auto_ptr<Glib::Error>& error)
-#endif
+bool ConnectionPool::drop_column(const Glib::ustring& table_name, const Glib::ustring& field_name) throw()
+{
+  sharedptr<SharedConnection> conn;
+  if(!m_refGdaConnection)
+  {
+    conn = connect();
+  }
+
+  if(!m_refGdaConnection)
+    return false;
+
+  try
+  {
+    m_backend->drop_column(m_refGdaConnection, table_name, field_name);
+    m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
+    return true;
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception:" << ex.what() << std::endl;
+  }
+
+  return false;
+}
+
+bool ConnectionPool::change_column(const Glib::ustring& table_name, const sharedptr<const Field>& field_old, const sharedptr<const Field>& field) throw()
 {
   type_vec_const_fields old_fields(1, field_old);
   type_vec_const_fields new_fields(1, field);
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   return change_columns(table_name, old_fields, new_fields);
-#else
-  return change_columns(table_name, old_fields, new_fields, error);
-#endif
 }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields)
-#else
-bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields, std::auto_ptr<Glib::Error>& error)
-#endif
+bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields) throw()
 {
   sharedptr<SharedConnection> conn;
   if(!m_refGdaConnection)
   {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     conn = connect();
-#else
-    std::auto_ptr<ExceptionConnection> local_error;
-    // TODO: We can't rethrow local_error here since ExceptionConnection does
-    // not derive from Glib::Error
-    conn = connect(local_error);
-#endif
   }
 
   if(!m_refGdaConnection)
     return false;
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  std::auto_ptr<Glib::Error> error;
-#endif
-  const bool result = m_backend->change_columns(m_refGdaConnection, table_name, old_fields, new_fields, error);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  if(error.get()) throw *error;
+  const bool result = m_backend->change_columns(m_refGdaConnection, table_name, old_fields, new_fields);
   m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
 
-#else
-  if(error.get())
-    std::cerr << "Error: " << error->what() << std::endl;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
-#endif
   return result;
 }
 
@@ -840,7 +732,7 @@ Document* ConnectionPool::get_document()
 {
   if(!m_slot_get_document)
   {
-    std::cerr << "Glom ConnectionPool::get_document(): m_slot_get_document is null." << std::endl;
+    std::cerr << G_STRFUNC << ": m_slot_get_document is null." << std::endl;
     return 0;
   }
 
@@ -860,7 +752,7 @@ EpcContents* ConnectionPool::on_publisher_document_requested(EpcPublisher* /* pu
     return 0;
 
   const Glib::ustring contents = document->get_contents();
-  //std::cout << "DEBUG: ConnectionPool::on_publisher_document_requested(): returning: " << std::endl << "  " << contents << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": returning: " << std::endl << "  " << contents << std::endl;
   return epc_contents_new_dup ("text/plain", (void*)contents.c_str(), -1);
 }
 
@@ -879,22 +771,22 @@ gboolean ConnectionPool::on_publisher_document_authentication(EpcAuthContext* co
   const gchar* password = epc_auth_context_get_password(context);
   g_return_val_if_fail(password, false); //TODO: This seems to happen once before this callback is called again properly.
 
-  //std::cout << "ConnectionPool::on_publisher_document_authentication(): username=" << user_name << ", password=" << password << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": username=" << user_name << ", password=" << password << std::endl;
 
   g_return_val_if_fail(connection_pool->m_backend.get(), false);
 
   //Attempt a connection with this username/password:
   std::auto_ptr<ExceptionConnection> error;
-  Glib::RefPtr<Gnome::Gda::Connection> connection = connection_pool->m_backend->connect(connection_pool->get_database(), user_name, password, error);
+  Glib::RefPtr<Gnome::Gda::Connection> connection = connection_pool->m_backend->connect(connection_pool->get_database(), user_name, password);
 
   if(connection)
   {
-    //std::cout << "ConnectionPool::on_publisher_document_authentication(): succeeded." << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": succeeded." << std::endl;
     return true; //Succeeded.
   }
   else
   {
-    //std::cout << "ConnectionPool::on_publisher_document_authentication(): failed." << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": failed." << std::endl;
     return false; //Failed.
   }
 }
@@ -986,7 +878,7 @@ void ConnectionPool::avahi_start_publishing()
   if(error)
   {
 #ifdef GLOM_CONNECTION_DEBUG
-    std::cout << "Glom: ConnectionPool::avahi_start_publishing(): Error while running epc_publisher_run_async: " << error->message << std::endl;
+    std::cout << "debug: " << G_STRFUNC << ": Error while running epc_publisher_run_async: " << error->message << std::endl;
 #endif
     g_clear_error(&error);
   }
@@ -1020,10 +912,15 @@ void ConnectionPool::set_show_debug_output(bool val)
 {
   m_show_debug_output = val;
 }
- 
+
 bool ConnectionPool::get_show_debug_output() const
 {
   return m_show_debug_output;
 }
 
+void ConnectionPool::set_auto_server_shutdown(bool val)
+{
+  m_auto_server_shutdown = val;
+}
+
 } //namespace Glom
