@@ -24,11 +24,7 @@
 #include <libglom/connectionpool_backends/postgres.h>
 #include <libglom/spawn_with_feedback.h>
 #include <libglom/utils.h>
-#include <libgdamm/config.h>
-#include <giomm/file.h>
-#include <glibmm/convert.h>
-#include <glibmm/miscutils.h>
-#include <glibmm/shell.h>
+#include <giomm.h>
 #include <glib/gstdio.h> /* For g_rename(). TODO: Wrap this in glibmm? */
 #include <glibmm/i18n.h>
 
@@ -67,7 +63,7 @@ float Postgres::get_postgres_server_version() const
   return m_postgres_server_version;
 }
 
-Glib::RefPtr<Gnome::Gda::Connection> Postgres::attempt_connect(const Glib::ustring& port, const Glib::ustring& database, const Glib::ustring& username, const Glib::ustring& password, std::auto_ptr<ExceptionConnection>& error) throw()
+Glib::RefPtr<Gnome::Gda::Connection> Postgres::attempt_connect(const Glib::ustring& port, const Glib::ustring& database, const Glib::ustring& username, const Glib::ustring& password)
 {
   //We must specify _some_ database even when we just want to create a database.
   //This _might_ be different on some systems. I hope not. murrayc
@@ -83,11 +79,10 @@ Glib::RefPtr<Gnome::Gda::Connection> Postgres::attempt_connect(const Glib::ustri
 
 #ifdef GLOM_CONNECTION_DEBUG
   std::cout << std::endl << "DEBUG: Glom: trying to connect on port=" << port << std::endl;
-  std::cout << "DEBUG: ConnectionPoolBackends::Postgres::attempt_connect(): cnc_string=" << cnc_string << std::endl;
+  std::cout << "debug: " << G_STRFUNC << ": cnc_string=" << cnc_string << std::endl;
   std::cout << "  DEBUG: auth_string=" << auth_string << std::endl;
 #endif
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     connection = Gnome::Gda::Connection::open_from_string("PostgreSQL",
@@ -97,65 +92,24 @@ Glib::RefPtr<Gnome::Gda::Connection> Postgres::attempt_connect(const Glib::ustri
     connection->statement_execute_non_select("SET DATESTYLE = 'ISO'");
     data_model = connection->statement_execute_select("SELECT version()");
   }
-  catch(const Gnome::Gda::ConfigError& ex)
-  {
-    //These errors are unusual.
-    //For instance, the PostgreSQL libgda provider could be missing,
-    //though we check for that at startup.
-    std::cerr << G_STRFUNC << 
-      ": ConfigError exception from Gnome::Gda::Connection::open_from_string(): " <<
-      ex.what();
-  }
   catch(const Glib::Error& ex)
   {
-#else
-  std::auto_ptr<Glib::Error> ex;
-  connection = Gnome::Gda::Connection::open_from_string("PostgreSQL",
-    cnc_string, auth_string,
-    Gnome::Gda::CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE,
-    ex);
-
-  if(!ex.get())
-    connection->statement_execute_non_select("SET DATESTYLE = 'ISO'", ex);
-
-  if(!ex.get())
-    data_model = connection->statement_execute_select("SELECT version()", Gnome::Gda::STATEMENT_MODEL_RANDOM_ACCESS, ex);
-
-  if(!ex.get())
-  {
-#endif
-
 #ifdef GLOM_CONNECTION_DEBUG
-    std::cout << "ConnectionPoolBackends::Postgres::attempt_connect(): Attempt to connect to database failed on port=" << port << ", database=" << database << ": " << ex.what() << std::endl;
-    std::cout << "ConnectionPoolBackends::Postgres::attempt_connect(): Attempting to connect without specifying the database." << std::endl;
+    std::cout << "debug: " << G_STRFUNC << ": Attempt to connect to database failed on port=" << port << ", database=" << database << ": " << ex.what() << std::endl;
+    std::cout << "debug: " << G_STRFUNC << ": Attempting to connect without specifying the database." << std::endl;
 #endif
 
-    Glib::ustring cnc_string = cnc_string_main + ";DB_NAME=" + default_database;
+    const Glib::ustring cnc_string = cnc_string_main + ";DB_NAME=" + default_database;
     Glib::RefPtr<Gnome::Gda::Connection> temp_conn;
     Glib::ustring auth_string = create_auth_string(username, password);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
       temp_conn = Gnome::Gda::Connection::open_from_string("PostgreSQL",
         cnc_string, auth_string,
         Gnome::Gda::CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
     }
-    catch(const Gnome::Gda::ConfigError& ex)
-    {
-      //These errors are unusual.
-      //For instance, the PostgreSQL libgda provider could be missing,
-      //though we check for that at startup.
-      std::cerr << G_STRFUNC << 
-        ": ConfigError exception from Gnome::Gda::Connection::open_from_string(): " <<
-        ex.what();
-    }
     catch(const Glib::Error& ex)
     {}
-#else
-    temp_conn = Gnome::Gda::Connection::open_from_string("PostgreSQL",
-      cnc_string, auth_string,
-      Gnome::Gda::CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE, ex);
-#endif
 
 #ifdef GLOM_CONNECTION_DEBUG
     if(temp_conn)
@@ -164,17 +118,13 @@ Glib::RefPtr<Gnome::Gda::Connection> Postgres::attempt_connect(const Glib::ustri
       std::cerr << "  (Could not connect even to the default database, database=" << database  << std::endl;
 #endif
 
-    error.reset(new ExceptionConnection(temp_conn ? ExceptionConnection::FAILURE_NO_DATABASE : ExceptionConnection::FAILURE_NO_SERVER));
+    throw ExceptionConnection(temp_conn ? ExceptionConnection::FAILURE_NO_DATABASE : ExceptionConnection::FAILURE_NO_SERVER);
     return Glib::RefPtr<Gnome::Gda::Connection>();
   }
 
   if(data_model && data_model->get_n_rows() && data_model->get_n_columns())
   {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     Gnome::Gda::Value value = data_model->get_value_at(0, 0);
-#else
-    Gnome::Gda::Value value = data_model->get_value_at(0, 0, ex);
-#endif
     if(value.get_value_type() == G_TYPE_STRING)
     {
       const Glib::ustring version_text = value.get_string();
@@ -196,249 +146,232 @@ Glib::RefPtr<Gnome::Gda::Connection> Postgres::attempt_connect(const Glib::ustri
   return connection;
 }
 
-bool Postgres::change_columns(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields, std::auto_ptr<Glib::Error>& error)
+bool Postgres::change_columns(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields) throw()
 {
   static const char TRANSACTION_NAME[] = "glom_change_columns_transaction";
   static const char TEMP_COLUMN_NAME[] = "glom_temp_column"; // TODO: Find a unique name.
 
-  if(!begin_transaction(connection, TRANSACTION_NAME, Gnome::Gda::TRANSACTION_ISOLATION_UNKNOWN, error)) return false; // TODO: What does the transaction isolation do?
-
-  for(unsigned int i = 0; i < old_fields.size(); ++ i)
-  {
-    // If the type did change, then we need to recreate the column. See
-    // http://www.postgresql.org/docs/faqs.FAQ.html#item4.3
-    if(old_fields[i]->get_field_info()->get_g_type() != new_fields[i]->get_field_info()->get_g_type())
-    {
-      // Create a temporary column
-      sharedptr<Field> temp_field = glom_sharedptr_clone(new_fields[i]);
-      temp_field->set_name(TEMP_COLUMN_NAME);
-      // The temporary column must not be primary key as long as the original
-      // (primary key) column is still present, because there cannot be two
-      // primary key columns.
-      temp_field->set_primary_key(false);
-
-      if(!add_column(connection, table_name, temp_field, error)) break;
-
-      Glib::ustring conversion_command;
-      const Glib::ustring field_name_old_quoted = "\"" + old_fields[i]->get_name() + "\"";
-      const Field::glom_field_type old_field_type = old_fields[i]->get_glom_type();
-
-      if(Field::get_conversion_possible(old_fields[i]->get_glom_type(), new_fields[i]->get_glom_type()))
-      {
-        //TODO: postgres seems to give an error if the data cannot be converted (for instance if the text is not a numeric digit when converting to numeric) instead of using 0.
-        /*
-        Maybe, for instance:
-        http://groups.google.de/groups?hl=en&lr=&ie=UTF-8&frame=right&th=a7a62337ad5a8f13&seekm=23739.1073660245%40sss.pgh.pa.us#link5
-        UPDATE _table
-        SET _bbb = to_number(substring(_aaa from 1 for 5), '99999')
-        WHERE _aaa <> '     ';
-        */
-
-        switch(new_fields[i]->get_glom_type())
-        {
-          case Field::TYPE_BOOLEAN:
-          {
-            if(old_field_type == Field::TYPE_NUMERIC)
-            {
-              conversion_command = "(CASE WHEN " + field_name_old_quoted + " > 0 THEN true "
-                                         "WHEN " + field_name_old_quoted + " = 0 THEN false "
-                                         "WHEN " + field_name_old_quoted + " IS NULL THEN false END)";
-            }
-            else if(old_field_type == Field::TYPE_TEXT)
-              conversion_command = '(' + field_name_old_quoted + " !~~* \'false\')"; // !~~* means ! ILIKE
-            else // Dates and Times:
-              conversion_command = '(' + field_name_old_quoted + " IS NOT NULL)";
-            break;
-          }
-
-          case Field::TYPE_NUMERIC: // CAST does not work if the destination type is numeric
-          {
-            if(old_field_type == Field::TYPE_BOOLEAN)
-            {
-              conversion_command = "(CASE WHEN " + field_name_old_quoted + " = true THEN 1 "
-                                         "WHEN " + field_name_old_quoted + " = false THEN 0 "
-                                         "WHEN " + field_name_old_quoted + " IS NULL THEN 0 END)";
-            }
-            else
-            {
-              //We use to_number, with textcat() so that to_number always has usable data.
-              //Otherwise, it says
-              //invalid input syntax for type numeric: " "
-              //
-              //We must use single quotes with the 0, otherwise it says "column 0 does not exist.".
-              conversion_command = "to_number( textcat(\'0\', " + field_name_old_quoted + "), '999999999.99999999' )";
-            }
-
-            break;
-          }
-
-          case Field::TYPE_DATE: // CAST does not work if the destination type is date.
-          {
-            conversion_command = "to_date( " + field_name_old_quoted + ", 'YYYYMMDD' )"; // TODO: Standardise date storage format.
-            break;
-          }
-          case Field::TYPE_TIME: // CAST does not work if the destination type is timestamp.
-          {
-            conversion_command = "to_timestamp( " + field_name_old_quoted + ", 'HHMMSS' )"; // TODO: Standardise time storage format.
-            break;
-          }
-
-          default:
-          {
-            // To Text:
-
-            // bool to text:
-            if(old_field_type == Field::TYPE_BOOLEAN)
-            {
-              conversion_command = "(CASE WHEN " + field_name_old_quoted + " = true THEN \'true\' "
-                                         "WHEN " + field_name_old_quoted + " = false THEN \'false\' "
-                                         "WHEN " + field_name_old_quoted + " IS NULL THEN \'false\' END)";
-            }
-            else
-            {
-              // This works for most to-text conversions:
-              conversion_command = "CAST(" + field_name_old_quoted + " AS " + new_fields[i]->get_sql_type() + ")";
-            }
-
-            break;
-          }
-        }
-
-        if(!query_execute(connection, "UPDATE \"" + table_name + "\" SET \"" + TEMP_COLUMN_NAME + "\" = " + conversion_command, error))
-          break;
-      }
-      else
-      {
-        // The conversion is not possible, so drop data in that column
-      }
-
-      if(!drop_column(connection, table_name, old_fields[i]->get_name(), error))
-        break;
-
-      if(!query_execute(connection, "ALTER TABLE \"" + table_name + "\" RENAME COLUMN \"" + TEMP_COLUMN_NAME + "\" TO \"" + new_fields[i]->get_name() + "\"", error))
-        break;
-
-      // Readd primary key constraint
-      if(new_fields[i]->get_primary_key())
-      {
-        if(!query_execute(connection, "ALTER TABLE \"" + table_name + "\" ADD PRIMARY KEY (\"" + new_fields[i]->get_name() + "\")", error))
-          break;
-      }
-    }
-    else
-    {
-      // The type did not change. What could have changed: The field being a
-      // unique key, primary key, its name or its default value.
-
-      // Primary key
-      // TODO: Test whether this is able to remove unique key constraints
-      // added via libgda's DDL API in add_column(). Maybe override
-      // add_column() if we can't.
-      bool primary_key_was_set = false;
-      bool primary_key_was_unset = false;
-      if(old_fields[i]->get_primary_key() != new_fields[i]->get_primary_key())
-      {
-        if(new_fields[i]->get_primary_key())
-        {
-          primary_key_was_set = true;
-
-          // Primary key was added
-          if(!query_execute(connection, "ALTER TABLE \"" + table_name + "\" ADD PRIMARY KEY (\"" + old_fields[i]->get_name() + "\")", error))
-            break;
-
-          // Remove unique key constraint, because this is already implied in
-          // the field being primary key.
-          //TODO: This depends on knowledge of the automatically-created 
-          //(during SERVER_OPERATION_CREATE_TABLE) constraint name.
-          //TODO: Find out how the table name and field name are escaped/quoted.
-          if(old_fields[i]->get_unique_key())
-            if(!query_execute(connection, "ALTER TABLE \"" + table_name + "\" DROP CONSTRAINT \"" + table_name + "_" + old_fields[i]->get_name() + "_key", error))
-              break;
-        }
-        else
-        {
-          primary_key_was_unset = true;
-
-          // Primary key was removed
-          if(!query_execute(connection, "ALTER TABLE \"" + table_name + "\" DROP CONSTRAINT \"" + table_name + "_pkey\"", error))
-            break;
-        }
-      }
-
-      // Uniqueness
-      if(old_fields[i]->get_unique_key() != new_fields[i]->get_unique_key())
-      {
-        // Postgres automatically makes primary keys unique, so we do not need
-        // to do that separately if we already made it a primary key
-        if(!primary_key_was_set && new_fields[i]->get_unique_key())
-        {
-          //TODO: This depends on knowledge of the automatically-created 
-          //(during SERVER_OPERATION_CREATE_TABLE) constraint name.
-          //TODO: Find out how the table name and field name are escaped/quoted.
-          if(!query_execute(connection, "ALTER TABLE \"" + table_name + "\" ADD CONSTRAINT \"" + table_name  + "_" + old_fields[i]->get_name() + "_key\" UNIQUE (\"" + old_fields[i]->get_name() + "\")", error))
-            break;
-        }
-        else if(!primary_key_was_unset && !new_fields[i]->get_unique_key() && !new_fields[i]->get_primary_key())
-        {
-          //TODO: This depends on knowledge of the automatically-created 
-          //(during SERVER_OPERATION_CREATE_TABLE) constraint name.
-          //TODO: Find out how the table name and field name are escaped/quoted.
-          if(!query_execute(connection, "ALTER TABLE \"" + table_name + "\" DROP CONSTRAINT \"" + table_name + "_" +old_fields[i]->get_name() + "_key\"", error))
-            break;
-        }
-      }
-
-      if(!new_fields[i]->get_auto_increment()) // Auto-increment fields have special code as their default values.
-      {
-        if(old_fields[i]->get_default_value() != new_fields[i]->get_default_value())
-        {
-          if(!query_execute(connection, "ALTER TABLE \"" + table_name + "\" ALTER COLUMN \"" + old_fields[i]->get_name() + "\" SET DEFAULT " + new_fields[i]->sql(new_fields[i]->get_default_value(), connection), error))
-            break;
-        }
-      }
-
-      if(old_fields[i]->get_name() != new_fields[i]->get_name())
-      {
-        if(!query_execute(connection, "ALTER TABLE \"" + table_name + "\" RENAME COLUMN \"" + old_fields[i]->get_name() + "\" TO \"" + new_fields[i]->get_name() + "\"", error))
-          break;
-      }
-    }
-  }
-
-  if(error.get() || !commit_transaction(connection, TRANSACTION_NAME, error))
-  {
-    std::auto_ptr<Glib::Error> rollback_error;
-    rollback_transaction(connection, TRANSACTION_NAME, rollback_error);
-    return false;
-  }
-
-  return true;
-}
-
-bool Postgres::attempt_create_database(const Glib::ustring& database_name, const Glib::ustring& host, const Glib::ustring& port, const Glib::ustring& username, const Glib::ustring& password, std::auto_ptr<Glib::Error>& error)
-{
-  Glib::RefPtr<Gnome::Gda::ServerOperation> op;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-    op = Gnome::Gda::ServerOperation::prepare_create_database("PostgreSQL",
-                                                              database_name);
+    connection->begin_transaction(TRANSACTION_NAME, Gnome::Gda::TRANSACTION_ISOLATION_UNKNOWN); // TODO: What does the transaction isolation do?
   }
   catch(const Glib::Error& ex)
   {
-    error.reset(new Glib::Error(ex));
-    return false;
+    std::cerr << G_STRFUNC << ": begin_transaction failed: " << ex.what() << std::endl;
   }
-#else
-  std::auto_ptr<Glib::Error> ex;
-  op = Gnome::Gda::ServerOperation::prepare_create_database("PostgreSQL",
-                                                            database_name,
-                                                            ex);
-  if(ex.get())
-    return false;
-#endif
+
+  //Do this all in one big try/catch, block, 
+  //reverting the transaction if anything fails:
+  try
+  {
+		for(unsigned int i = 0; i < old_fields.size(); ++ i)
+		{
+		  // If the type did change, then we need to recreate the column. See
+		  // http://www.postgresql.org/docs/faqs.FAQ.html#item4.3
+		  if(old_fields[i]->get_field_info()->get_g_type() != new_fields[i]->get_field_info()->get_g_type())
+		  {
+		    // Create a temporary column
+		    sharedptr<Field> temp_field = glom_sharedptr_clone(new_fields[i]);
+		    temp_field->set_name(TEMP_COLUMN_NAME);
+		    // The temporary column must not be primary key as long as the original
+		    // (primary key) column is still present, because there cannot be two
+		    // primary key columns.
+		    temp_field->set_primary_key(false);
+
+		    add_column(connection, table_name, temp_field);
+
+		    Glib::ustring conversion_command;
+		    const Glib::ustring field_name_old_quoted = "\"" + old_fields[i]->get_name() + "\"";
+		    const Field::glom_field_type old_field_type = old_fields[i]->get_glom_type();
+
+		    if(Field::get_conversion_possible(old_fields[i]->get_glom_type(), new_fields[i]->get_glom_type()))
+		    {
+		      //TODO: postgres seems to give an error if the data cannot be converted (for instance if the text is not a numeric digit when converting to numeric) instead of using 0.
+		      /*
+		      Maybe, for instance:
+		      http://groups.google.de/groups?hl=en&lr=&ie=UTF-8&frame=right&th=a7a62337ad5a8f13&seekm=23739.1073660245%40sss.pgh.pa.us#link5
+		      UPDATE _table
+		      SET _bbb = to_number(substring(_aaa from 1 for 5), '99999')
+		      WHERE _aaa <> '     ';  
+		      */
+
+		      switch(new_fields[i]->get_glom_type())
+		      {
+		        case Field::TYPE_BOOLEAN:
+		        {
+		          if(old_field_type == Field::TYPE_NUMERIC)
+		          {
+		            conversion_command = "(CASE WHEN " + field_name_old_quoted + " > 0 THEN true "
+		                                       "WHEN " + field_name_old_quoted + " = 0 THEN false "
+		                                       "WHEN " + field_name_old_quoted + " IS NULL THEN false END)";
+		          }
+		          else if(old_field_type == Field::TYPE_TEXT)
+		            conversion_command = '(' + field_name_old_quoted + " !~~* \'false\')"; // !~~* means ! ILIKE
+		          else // Dates and Times:
+		            conversion_command = '(' + field_name_old_quoted + " IS NOT NULL)";
+		          break;
+		        }
+
+		        case Field::TYPE_NUMERIC: // CAST does not work if the destination type is numeric
+		        {
+		          if(old_field_type == Field::TYPE_BOOLEAN)
+		          {
+		            conversion_command = "(CASE WHEN " + field_name_old_quoted + " = true THEN 1 "
+		                                       "WHEN " + field_name_old_quoted + " = false THEN 0 "
+		                                       "WHEN " + field_name_old_quoted + " IS NULL THEN 0 END)";
+		          }
+		          else
+		          {
+		            //We use to_number, with textcat() so that to_number always has usable data.
+		            //Otherwise, it says 
+		            //invalid input syntax for type numeric: " "
+		            //
+		            //We must use single quotes with the 0, otherwise it says "column 0 does not exist.".
+		            conversion_command = "to_number( textcat(\'0\', " + field_name_old_quoted + "), '999999999.99999999' )";
+		          }
+
+		          break;
+		        }
+
+		        case Field::TYPE_DATE: // CAST does not work if the destination type is date.
+		        {
+		          conversion_command = "to_date( " + field_name_old_quoted + ", 'YYYYMMDD' )"; // TODO: Standardise date storage format.
+		          break;
+		        }
+		        case Field::TYPE_TIME: // CAST does not work if the destination type is timestamp.
+		        {
+		          conversion_command = "to_timestamp( " + field_name_old_quoted + ", 'HHMMSS' )"; // TODO: Standardise time storage format.
+		          break;
+		        }
+
+		        default:
+		        {
+		          // To Text:
+
+		          // bool to text:
+		          if(old_field_type == Field::TYPE_BOOLEAN)
+		          {
+		            conversion_command = "(CASE WHEN " + field_name_old_quoted + " = true THEN \'true\' "
+		                                       "WHEN " + field_name_old_quoted + " = false THEN \'false\' "
+		                                       "WHEN " + field_name_old_quoted + " IS NULL THEN \'false\' END)";
+		          }
+		          else
+		          {
+		            // This works for most to-text conversions:
+		            conversion_command = "CAST(" + field_name_old_quoted + " AS " + new_fields[i]->get_sql_type() + ")";
+		          }
+
+		          break;
+		        }
+		      }
+
+		      connection->statement_execute_non_select("UPDATE \"" + table_name + "\" SET \"" + TEMP_COLUMN_NAME + "\" = " + conversion_command);
+		    }
+		    else
+		    {
+		      // The conversion is not possible, so drop data in that column
+		    }
+
+		    drop_column(connection, table_name, old_fields[i]->get_name());
+
+		    connection->statement_execute_non_select("ALTER TABLE \"" + table_name + "\" RENAME COLUMN \"" + TEMP_COLUMN_NAME + "\" TO \"" + new_fields[i]->get_name() + "\"");
+
+		    // Read primary key constraint
+		    if(new_fields[i]->get_primary_key())
+		    {
+		      connection->statement_execute_non_select("ALTER TABLE \"" + table_name + "\" ADD PRIMARY KEY (\"" + new_fields[i]->get_name() + "\")");
+		    }
+		  }
+		  else
+		  {
+		    // The type did not change. What could have changed: The field being a
+		    // unique key, primary key, its name or its default value.
+
+		    // Primary key
+		    // TODO: Test whether this is able to remove unique key constraints
+		    // added via libgda's DDL API in add_column(). Maybe override
+		    // add_column() if we can't.
+		    bool primary_key_was_set = false;
+		    bool primary_key_was_unset = false;
+		    if(old_fields[i]->get_primary_key() != new_fields[i]->get_primary_key())
+		    {
+		      if(new_fields[i]->get_primary_key())
+		      {
+		        primary_key_was_set = true;
+
+		        // Primary key was added
+		       connection->statement_execute_non_select("ALTER TABLE \"" + table_name + "\" ADD PRIMARY KEY (\"" + old_fields[i]->get_name() + "\")");
+
+		        // Remove unique key constraint, because this is already implied in
+		        // the field being primary key.
+		        if(old_fields[i]->get_unique_key())
+		          connection->statement_execute_non_select("ALTER TABLE \"" + table_name + "\" DROP CONSTRAINT \"" + old_fields[i]->get_name() + "_key");
+		      }
+		      else
+		      {
+		        primary_key_was_unset = true;
+
+		        // Primary key was removed
+		        connection->statement_execute_non_select("ALTER TABLE \"" + table_name + "\" DROP CONSTRAINT \"" + table_name + "_pkey\"");
+		      }
+		    }
+
+		    // Uniqueness
+		    if(old_fields[i]->get_unique_key() != new_fields[i]->get_unique_key())
+		    {
+		      // Postgres automatically makes primary keys unique, so we do not need
+		      // to do that separately if we already made it a primary key
+		      if(!primary_key_was_set && new_fields[i]->get_unique_key())
+		      {
+		        connection->statement_execute_non_select("ALTER TABLE \"" + table_name + "\" ADD CONSTRAINT \"" + old_fields[i]->get_name() + "_key\" UNIQUE (\"" + old_fields[i]->get_name() + "\")");
+		      }
+		      else if(!primary_key_was_unset && !new_fields[i]->get_unique_key() && !new_fields[i]->get_primary_key())
+		      {
+		        connection->statement_execute_non_select("ALTER TABLE \"" + table_name + "\" DROP CONSTRAINT \"" + old_fields[i]->get_name() + "_key\"");
+		      }
+		    }
+
+		    if(!new_fields[i]->get_auto_increment()) // Auto-increment fields have special code as their default values.
+		    {
+		      if(old_fields[i]->get_default_value() != new_fields[i]->get_default_value())
+		      {
+		        connection->statement_execute_non_select("ALTER TABLE \"" + table_name + "\" ALTER COLUMN \"" + old_fields[i]->get_name() + "\" SET DEFAULT " + new_fields[i]->sql(new_fields[i]->get_default_value(), connection));
+		      }
+		    }
+
+		    if(old_fields[i]->get_name() != new_fields[i]->get_name())
+		    {
+		      connection->statement_execute_non_select("ALTER TABLE \"" + table_name + "\" RENAME COLUMN \"" + old_fields[i]->get_name() + "\" TO \"" + new_fields[i]->get_name() + "\"");
+		    }
+		  }
+		}
+		
+		connection->commit_transaction(TRANSACTION_NAME);
+		return true;
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << "Exception: " << ex.what() << std::endl;
+    std::cerr << "Reverting the transaction." << std::endl;
+    
+    try
+    {
+      connection->rollback_transaction(TRANSACTION_NAME);
+    }
+    catch(const Glib::Error& ex)
+    {
+      std::cerr << "Could not rollback the transaction: Exception: " << ex.what() << std::endl;
+    }
+  }
+  
+  return false;
+}
+
+bool Postgres::attempt_create_database(const Glib::ustring& database_name, const Glib::ustring& host, const Glib::ustring& port, const Glib::ustring& username, const Glib::ustring& password)
+{
+  Glib::RefPtr<Gnome::Gda::ServerOperation> op = Gnome::Gda::ServerOperation::prepare_create_database("PostgreSQL",
+                                                              database_name);
   g_assert(op);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     op->set_value_at("/SERVER_CNX_P/HOST", host);
@@ -449,22 +382,9 @@ bool Postgres::attempt_create_database(const Glib::ustring& database_name, const
   }
   catch(const Glib::Error& ex)
   {
-    error.reset(new Glib::Error(ex));
+    std::cerr << G_STRFUNC << ": exception: " << ex.what() << std::endl;
     return false;
   }
-#else //GLIBMM_EXCEPTIONS_ENABLED
-  op->set_value_at("/SERVER_CNX_P/HOST", host, error);
-  op->set_value_at("/SERVER_CNX_P/PORT", port, error);
-  op->set_value_at("/SERVER_CNX_P/ADM_LOGIN", username, error);
-  op->set_value_at("/SERVER_CNX_P/ADM_PASSWORD", password, error);
-
-  if(error.get() == 0)
-    op->perform_create_database("PostgreSQL", ex);
-  else
-    return false;
-  if (error.get())
-    return false;
-#endif //GLIBMM_EXCEPTIONS_ENABLED
 
   return true;
 }
@@ -497,7 +417,7 @@ bool Postgres::check_postgres_gda_client_is_available()
   return false;
 }
 
-std::string Postgres::get_path_to_postgres_executable(const std::string& program, bool quoted)
+std::string Postgres::get_path_to_postgres_executable(const std::string& program)
 {
 #ifdef G_OS_WIN32
   // Add the .exe extension on Windows:
@@ -526,25 +446,15 @@ std::string Postgres::get_path_to_postgres_executable(const std::string& program
   g_free(installation_directory);
 
   if(Glib::file_test(test, Glib::FILE_TEST_IS_EXECUTABLE))
-  {
-    if(quoted)
-      test = Glib::shell_quote(path);
     return test;
-  }
 
   // Look in PATH otherwise
-  std::string path = Glib::find_program_in_path(real_program);
-  if(quoted)
-    path = Glib::shell_quote(path);
-  return path;
+  return Glib::find_program_in_path(real_program);
 #else // G_OS_WIN32
   // POSTGRES_UTILS_PATH is defined in config.h, based on the configure.
   try
   {
-    std::string path = Glib::build_filename(POSTGRES_UTILS_PATH, program + EXEEXT);
-    if(quoted)
-      path = Glib::shell_quote(path);
-    return path;
+    return Glib::build_filename(POSTGRES_UTILS_PATH, program + EXEEXT);
   }
   catch(const Glib::Error& ex)
   {
@@ -673,13 +583,13 @@ bool Postgres::save_backup(const SlotProgress& slot_progress, const Glib::ustrin
 
   // Make sure to use double quotes for the executable path, because the
   // CreateProcess() API used on Windows does not support single quotes.
-  const std::string command_dump = get_path_to_postgres_executable("pg_dump") +
+  const std::string command_dump = "\"" + get_path_to_postgres_executable("pg_dump") + "\"" +
     " --format=c " + // The default (plain) format cannot be used with pg_restore.
-    " --create --file=" + Glib::shell_quote(path_backup) +
-    " --host=" + Glib::shell_quote(m_host) +
+    " --create --file=\"" + path_backup + "\"" +
+    " --host=\"" + m_host + "\"" +
     " --port=" + port_as_string(m_port) +
-    " --username=" + Glib::shell_quote(username) +
-    " " + database_name; //TODO: Quote database_name?
+    " --username=\"" + username + "\"" +
+    " " + database_name;
 
 
   //std::cout << "DEBUG: command_dump=" << command_dump << std::endl;
@@ -757,11 +667,11 @@ bool Postgres::convert_backup(const SlotProgress& slot_progress, const std::stri
 
   // Make sure to use double quotes for the executable path, because the
   // CreateProcess() API used on Windows does not support single quotes.
-  const std::string command_restore = get_path_to_postgres_executable("pg_restore") +
-    " -d " + database_name + //TODO: Quote database name?
-    " --host=" + Glib::shell_quote(m_host) +
+  const std::string command_restore = "\"" + get_path_to_postgres_executable("pg_restore") + "\"" +
+    " -d " + database_name +
+    " --host=\"" + m_host + "\"" +
     " --port=" + port_as_string(m_port) +
-    " --username=" + Glib::shell_quote(username) +
+    " --username=\"" + username + "\"" +
     " " + path_backup;
 
   std::cout << "DEBUG: command_restore=" << command_restore << std::endl;
