@@ -231,7 +231,7 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnom
   Glib::RefPtr<const Gnome::Gda::ServerProvider> provider = connection->get_provider();
   if(!provider)
   {
-    std::cerr << "Field::sql(): The ServerProvider was null." << std::endl;
+    std::cerr << G_STRFUNC << ": The ServerProvider was null." << std::endl;
     return Glib::ustring();
   } 
 
@@ -246,7 +246,7 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnom
   }
   else
   {
-    std::cerr << "Field::sql(): The DataHandler was null." << std::endl;
+    std::cerr << G_STRFUNC << ": The DataHandler was null." << std::endl;
     return Glib::ustring();
   }
 
@@ -256,7 +256,6 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnom
 Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
 {
   sharedptr<SharedConnection> connection;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     connection = ConnectionPool::get_instance()->connect();
@@ -266,12 +265,6 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
     //TODO: Do something more useful here
     std::cerr << "Field::sql, connection failed: " << ex.what() << std::endl;
   }
-#else
-  std::auto_ptr<Glom::ExceptionConnection> ex;
-  connection = ConnectionPool::get_instance()->connect(ex);
-  if (ex.get())
-    std::cerr << "Field::sql, connection failed: " << ex->what() << std::endl;
-#endif
   if(connection)
   {
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = connection->get_gda_connection();
@@ -382,7 +375,7 @@ Glib::ustring Field::sql_find(const Gnome::Gda::Value& value) const
       if(value.is_null())
         return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
       else
-        return ("%" + value.to_string() + "%"); //TODO: Escape it before adding "%", but then prevent SqlBuilder from re-escaping it.
+        return ("'%" + value.to_string() + "%'"); //Add single-quotes. TODO: Actually escape it.
         
       break;
     }
@@ -398,7 +391,7 @@ Glib::ustring Field::sql_find(const Gnome::Gda::Value& value) const
   }
 }
 
-Gnome::Gda::SqlOperatorType Field::sql_find_operator() const
+Glib::ustring Field::sql_find_operator() const
 {
   switch(get_glom_type())
   {
@@ -408,7 +401,7 @@ Gnome::Gda::SqlOperatorType Field::sql_find_operator() const
       if(connection_pool && connection_pool->get_backend())
         return connection_pool->get_string_find_operator();
       else
-        return Gnome::Gda::SQL_OPERATOR_TYPE_LIKE; // Default
+        return "LIKE"; // Default
       break;
     }
     case(TYPE_DATE):
@@ -417,7 +410,7 @@ Gnome::Gda::SqlOperatorType Field::sql_find_operator() const
     case(TYPE_BOOLEAN):
     default:
     {
-      return Gnome::Gda::SQL_OPERATOR_TYPE_EQ;
+      return "=";
       break;
     }
   }
@@ -551,17 +544,12 @@ Glib::RefPtr<Gnome::Gda::Holder> Field::get_holder(const Gnome::Gda::Value& valu
     // investigate why the field type is not GdaBinary as well.
     // Maybe get_gda_type_for_glom_type() should already return fallback
     // types if necessary.
-    std::cout << "DEBUG: Field::get_holder(): Field type " << g_type_name(field_type) << " and value type " << g_type_name(gtype) << " don't match." << std::endl;
+    std::cout << "debug: " << G_STRFUNC << ": Field type " << g_type_name(field_type) << " and value type " << g_type_name(gtype) << " don't match." << std::endl;
   }
   */
 
   Glib::RefPtr<Gnome::Gda::Holder> holder = Gnome::Gda::Holder::create(gtype, real_name);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   holder->set_value_as_value(value);
-#else
-  std::auto_ptr<Glib::Error> ex;
-  holder->set_value_as_value(value, ex);
-#endif
   return holder;
 }
 
@@ -599,7 +587,7 @@ Field::glom_field_type Field::get_glom_type_for_gda_type(GType gda_type)
       result = iterFind->second;
     else
     {
-      std::cerr << G_STRFUNC << ": Unhandled GType: " << g_type_name(gda_type) << std::endl;
+      // g_warning("FieldType::FieldType(GType gda_type): Invalid gda type: %d",  gda_type);
     }
   }
 
@@ -621,7 +609,7 @@ GType Field::get_gda_type_for_glom_type(Field::glom_field_type glom_type)
     g_warning("Field::get_gda_type_for_glom_type(): Returning G_TYPE_NONE for glom_type=%d", glom_type);
   }
 
-  //std::cout << "Field::get_gda_type_for_glom_type(): returning: " << g_type_name(ideal_gda_type) << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": returning: " << g_type_name(ideal_gda_type) << std::endl;
   
   return ideal_gda_type;
 }
@@ -642,11 +630,6 @@ void Field::init_map()
     m_map_gda_type_to_glom_type[G_TYPE_BOOLEAN] = TYPE_BOOLEAN;
     m_map_gda_type_to_glom_type[GDA_TYPE_BINARY] = TYPE_IMAGE;
 
-    //Extra conversions for GTypes that can be returned by glom_pygda_value_from_pyobject():
-    m_map_gda_type_to_glom_type[G_TYPE_DOUBLE] = TYPE_NUMERIC;
-    //TODO? m_map_gda_type_to_glom_type[GDA_TYPE_TIME] = ;
-    //TODO? m_map_gda_type_to_glom_type[GDA_TYPE_TIMESTAMP] = ;
-
     m_map_glom_type_to_gda_type[TYPE_NUMERIC] = GDA_TYPE_NUMERIC;
     m_map_glom_type_to_gda_type[TYPE_TEXT] = G_TYPE_STRING;
     m_map_glom_type_to_gda_type[TYPE_TIME] = GDA_TYPE_TIME;
