@@ -18,8 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef DOCUMENT_GLOM_H
-#define DOCUMENT_GLOM_H
+#ifndef GLOM_DOCUMENT_H
+#define GLOM_DOCUMENT_H
 
 #include <libglom/libglom_config.h>
 
@@ -127,7 +127,7 @@ public:
 
   void set_connection_server(const Glib::ustring& strVal);
   void set_connection_database(const Glib::ustring& strVal);
-  void set_connection_port(int port_number);
+  void set_connection_port(unsigned int port_number);
   void set_connection_try_other_ports(bool val);
 
  /** Temporarily set a username in the document.
@@ -147,7 +147,7 @@ public:
 
   Glib::ustring get_connection_server() const;
   Glib::ustring get_connection_database() const;
-  int get_connection_port() const;
+  unsigned int get_connection_port() const;
   bool get_connection_try_other_ports() const;
 
   /** Retrieve a username previously set in the document.
@@ -514,7 +514,7 @@ private:
 
   Glib::ustring m_connection_server, m_connection_database;
   Glib::ustring m_connection_user; //Don't save the user.
-  int m_connection_port; //0 means any port. Ignored when self-hosting (which may use a different port each time).
+  unsigned int m_connection_port; //0 means any port. Ignored when self-hosting (which may use a different port each time).
   bool m_connection_try_other_ports; //Set to false for self-hosted or browsed-from-network documents.
 
   class LayoutInfo
@@ -631,4 +631,4 @@ private:
 
 } //namespace Glom
 
-#endif //DOCUMENT_GLOM_H
+#endif // GLOM_DOCUMENT_H
