@@ -26,7 +26,9 @@
 #include <libglom/document/document.h>
 
 #include <libglom/data_structure/layout/layoutitem_field.h>
+
 #include <libgdamm/sqlexpr.h>
+#include <giomm/file.h>
 
 namespace Glom
 {
@@ -40,7 +42,7 @@ namespace Utils
 
 Glib::ustring trim_whitespace(const Glib::ustring& text);
 
-Glib::ustring string_replace(const Glib::ustring& src, const Glib::ustring search_for, const Glib::ustring& replace_with);
+Glib::ustring string_replace(const Glib::ustring& src, const Glib::ustring& search_for, const Glib::ustring& replace_with);
 
 //typedef Base_DB::type_vecLayoutFields type_vecLayoutFields;
 typedef std::vector< sharedptr<LayoutItem_Field> > type_vecLayoutFields;
@@ -86,11 +88,15 @@ Glib::RefPtr<Gnome::Gda::SqlBuilder> build_sql_select_with_where_clause(
   const type_sort_clause& sort_clause = type_sort_clause(),
   guint limit = 0);
 
+/**
+ * @param key_value If this is empty then all records in the tables will be retrieved.
+ */
 Glib::RefPtr<Gnome::Gda::SqlBuilder> build_sql_select_with_key(
   const Glib::ustring& table_name,
   const type_vecLayoutFields& fieldsToGet,
   const sharedptr<const Field>& key_field,
   const Gnome::Gda::Value& key_value,
+  const type_sort_clause& sort_clause = type_sort_clause(),
   guint limit = 0);
 
 /** Just a version of build_sql_select_with_key() that takes a list of const fields.
@@ -100,13 +106,16 @@ Glib::RefPtr<Gnome::Gda::SqlBuilder> build_sql_select_with_key(
   const type_vecConstLayoutFields& fieldsToGet,
   const sharedptr<const Field>& key_field,
   const Gnome::Gda::Value& key_value,
+  const type_sort_clause& sort_clause = type_sort_clause(),
   guint limit = 0);
 
-Gnome::Gda::SqlExpr get_find_where_clause_quick(Document* document, const Glib::ustring& table_name, const Gnome::Gda::Value& quick_search);
+Gnome::Gda::SqlExpr get_find_where_clause_quick(const Document* document, const Glib::ustring& table_name, const Gnome::Gda::Value& quick_search);
 
+typedef std::list<Gnome::Gda::Value> type_list_values;
+typedef std::list< std::pair<Gnome::Gda::Value, type_list_values> > type_list_values_with_second; //TODO: Rename this now that we have more than just 1 extra field.
+type_list_values_with_second get_choice_values_all(const Document* document, const sharedptr<const LayoutItem_Field>& field);
 
-typedef std::list< std::pair<Gnome::Gda::Value, Gnome::Gda::Value> > type_list_values_with_second;
-type_list_values_with_second get_choice_values(const sharedptr<const LayoutItem_Field>& field);
+type_list_values_with_second get_choice_values(const Document* document, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& foreign_key_value);
 
 /// Get the full query string suitable for use with std::cout.
 std::string sqlbuilder_get_full_query(
@@ -154,6 +163,28 @@ Glib::ustring string_remove_suffix(const Glib::ustring& str, const Glib::ustring
 
 bool file_exists(const Glib::ustring& uri);
 
+/** Delete a directory, if it exists, and its contents.
+ * Unlike g_file_delete(), this does not fail if the directory is not empty.
+ */
+bool delete_directory(const Glib::RefPtr<Gio::File>& directory);
+
+/** Delete a directory, if it exists, and its contents.
+ * Unlike g_file_delete(), this does not fail if the directory is not empty.
+ */
+bool delete_directory(const std::string& uri);
+
+/** For instance, to find the first file in the directory with a .glom extension.
+ */
+Glib::ustring get_directory_child_with_suffix(const Glib::ustring& uri_directory, const std::string& suffix, bool recursive);
+
+/** Get a string to display to the user, as a representation of a list of layout items.
+ */
+Glib::ustring get_list_of_layout_items_for_display(const LayoutGroup::type_list_items& list_layout_fields);
+
+/** Get a string to display to the user, as a representation of a list of layout items.
+ */
+Glib::ustring get_list_of_layout_items_for_display(const sharedptr<const LayoutGroup>& layout_group);
+
 } //namespace Utils
 
 } //namespace Glom
