@@ -25,7 +25,6 @@
 #include <libglom/data_structure/glomconversions.h>
 #include <libglom/db_utils.h>
 #include <glom/frame_glom.h> //For show_ok_dialog()
-#include <gtkmm/stock.h>
 #include <glom/glade_utils.h>
 #include <glibmm/i18n.h>
 
@@ -76,7 +75,7 @@ bool Box_Data_Calendar_Related::init_db_details(const sharedptr<const LayoutItem
 
 bool Box_Data_Calendar_Related::init_db_details(const Glib::ustring& parent_table, bool show_title)
 {
-  //std::cout << "DEBUG: Box_Data_Calendar_Related::init_db_details(): " << parent_table << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": " << parent_table << std::endl;
 
   m_parent_table = parent_table;
 
@@ -90,11 +89,16 @@ bool Box_Data_Calendar_Related::init_db_details(const Glib::ustring& parent_tabl
   //TODO: This is duplicated in box_data_related_list.cc and box_data_portal.cc. Just use code from the base class?
   if(show_title)
   {
-    Glib::ustring title;
-    if(m_portal)
-      title = m_portal->get_title();
+    Glib::ustring relationship_title;
+    if(m_portal && m_portal->get_has_relationship_name())
+      relationship_title = m_portal->get_title_used(Glib::ustring() /* parent title - not relevant */);
+    else
+    {
+      //Note to translators: This text is shown instead of a table title, when the table has not yet been chosen.
+      relationship_title = _("Undefined Table");
+    }
 
-    m_Label.set_markup(Utils::bold_message(title));
+    m_Label.set_markup(Utils::bold_message(relationship_title));
     m_Label.show();
 
     m_Alignment.set_padding(Utils::DEFAULT_SPACING_SMALL /* top */, 0, Utils::DEFAULT_SPACING_LARGE /* left */, 0);
@@ -119,14 +123,6 @@ bool Box_Data_Calendar_Related::init_db_details(const Glib::ustring& parent_tabl
   return Box_Data::init_db_details(found_set, "" /* layout_platform */); //Calls create_layout() and fill_from_database().
 }
 
-
-void Box_Data_Calendar_Related::create_layout()
-{
-  Box_Data::create_layout();
-
-  m_FieldsShown = get_fields_to_show();
-}
-
 bool Box_Data_Calendar_Related::fill_from_database()
 {
   if(!m_portal)
@@ -218,26 +214,14 @@ bool Box_Data_Calendar_Related::fill_from_database()
        continue;
 
       //Get the date value for this row:
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
       Gnome::Gda::Value value_date = datamodel->get_value_at(m_query_column_date_field, row_index);
-#else
-      std::auto_ptr<Glib::Error> error;
-      Gnome::Gda::Value value_date = datamodel->get_value_at(m_query_column_date_field, row_index, error);
-#endif
       const Glib::Date date = value_date.get_date();
 
       //Get all the values for this row:
       type_vector_values* pVector = new type_vector_values(m_FieldsShown.size());
       for(int column_index = 0; column_index < columns_count; ++column_index)
       {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
         (*pVector)[column_index] = datamodel->get_value_at(column_index, row_index);
-#else
-        std::auto_ptr<Glib::Error> error;
-          (*pVector)[column_index] = datamodel->get_value_at(column_index, row_index, error);
-        if (error.get())
-          break;
-#endif
       }
 
       m_map_values[date].push_back(pVector);
@@ -326,23 +310,17 @@ void Box_Data_Calendar_Related::on_record_added(const Gnome::Gda::Value& primary
   }
 }
 
-Box_Data_Calendar_Related::type_vecLayoutFields Box_Data_Calendar_Related::get_fields_to_show() const
+Box_Data_Calendar_Related::type_vecConstLayoutFields Box_Data_Calendar_Related::get_fields_to_show() const
 {
-  type_vecLayoutFields layout_fields = Box_Data_Portal::get_fields_to_show();
+  type_vecConstLayoutFields layout_fields = Box_Data_Portal::get_fields_to_show();
 
   sharedptr<LayoutItem_CalendarPortal> derived_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(m_portal);
   if(!derived_portal)
-  {
-    std::cerr << G_STRFUNC << ": The portal is not a LayoutItem_CalendarPortal." << std::endl;
     return layout_fields;
-  }
 
   sharedptr<const Field> date_field = derived_portal->get_date_field();
   if(!date_field)
-  {
-    std::cerr << G_STRFUNC << ": get_date_field() returned no field." << std::endl;
     return layout_fields;
-  }
 
   //Add it to the list to ensure that we request the date (though it will not really be shown in the calendar):
   sharedptr<LayoutItem_Field> layout_item_date_field = sharedptr<LayoutItem_Field>::create();
@@ -414,19 +392,16 @@ Glib::ustring Box_Data_Calendar_Related::on_calendar_details(guint year, guint m
   sharedptr<LayoutItem_CalendarPortal> derived_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(m_portal);
   if(!derived_portal)
   {
-    //std::cout << "DEBUG: Box_Data_Calendar_Related::on_calendar_details(): date_field is NULL" << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": date_field is NULL" << std::endl;
     return Glib::ustring();
   }
 
   sharedptr<const Field> date_field = derived_portal->get_date_field();
   if(!date_field)
-  {
-    std::cerr << G_STRFUNC << ":  get_date_field() returned no field." << std::endl;
     return Glib::ustring();
-  }
 
   //TODO: month seems to be 143710360 sometimes, which seems to be a GtkCalendar bug:
-  //std::cout << "Box_Data_Calendar_Related::on_calendar_details(): year=" << year << ", month=" << month << " day=" << day << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": year=" << year << ", month=" << month << " day=" << day << std::endl;
 
   //Glib::Date is 1-indexed:
   Glib::Date::Month datemonth = (Glib::Date::Month)(month +1);
@@ -537,10 +512,8 @@ void Box_Data_Calendar_Related::setup_menu()
 
   //TODO: add_accel_group(m_refUIManager->get_accel_group());
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-#endif
     Glib::ustring ui_info =
         "<ui>"
         "  <popup name='ContextMenu'>"
@@ -551,21 +524,12 @@ void Box_Data_Calendar_Related::setup_menu()
         "  </popup>"
         "</ui>";
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     m_refUIManager->add_ui_from_string(ui_info);
   }
   catch(const Glib::Error& ex)
   {
     std::cerr << "building menus failed: " <<  ex.what();
   }
-#else
-  std::auto_ptr<Glib::Error> error;
-  m_refUIManager->add_ui_from_string(ui_info, error);
-  if(error.get())
-  {
-    std::cerr << "building menus failed: " << error->what();
-  }
-#endif //GLIBMM_EXCEPTIONS_ENABLED
 
   //Get the menu:
   m_pMenuPopup = dynamic_cast<Gtk::Menu*>( m_refUIManager->get_widget("/ContextMenu") );
