@@ -31,11 +31,6 @@
 #include <libglom/privs.h>
 #include <glom/xsl_utils.h>
 #include <glom/python_embed/glom_python.h>
-
-#include <glom/application.h>
-#include <gtkmm/stock.h>
-#include <gtkmm/viewport.h>
-
 #include <sstream> //For stringstream
 #include <glibmm/i18n.h>
 
@@ -63,14 +58,14 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 
   add_view(&m_FlowTable); //Allow this to access the document too.
 
-  m_FlowTable.set_columns_count(1); //Sub-groups will have multiple columns (by default, there is one sub-group, with 2 columns).
+  m_FlowTable.set_lines(1); //Sub-groups will have multiple columns (by default, there is one sub-group, with 2 columns).
 
   #ifndef GLOM_ENABLE_MAEMO
-  m_FlowTable.set_column_padding(Utils::DEFAULT_SPACING_SMALL); //The default anyway.
-  m_FlowTable.set_row_padding(Utils::DEFAULT_SPACING_SMALL); //The default anyway.
+  m_FlowTable.set_horizontal_spacing(Utils::DEFAULT_SPACING_SMALL); //The default anyway.
+  m_FlowTable.set_vertical_spacing(Utils::DEFAULT_SPACING_SMALL); //The default anyway.
   #else
-  m_FlowTable.set_row_padding(0); //The hildon buttons and entries have their own default padding.
-  m_FlowTable.set_column_padding(HILDON_MARGIN_DOUBLE); //As per the UI specs.
+  m_FlowTable.set_vertical_spacing(0); //The hildon buttons and entries have their own default padding.
+  m_FlowTable.set_horizontal_spacing(HILDON_MARGIN_DOUBLE); //As per the UI specs.
   #endif
 
   //m_strHint = _("When you change the data in a field the database is updated immediately.\n Click [New] to add a new record.\n Leave automatic ID fields empty - they will be filled for you.");
@@ -186,13 +181,13 @@ void Box_Data_Details::set_found_set_from_primary_key_value()
   {
     m_found_set.m_where_clause = Utils::build_simple_where_expression(
        m_table_name, m_field_primary_key, m_primary_key_value);
-    //std::cout << "  DEBUG: Box_Data_Details::set_primary_key_value(): m_found_set.m_where_clause = " << m_found_set.m_where_clause << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": m_found_set.m_where_clause = " << m_found_set.m_where_clause << std::endl;
   }
 }
 
 bool Box_Data_Details::init_db_details(const FoundSet& found_set, const Glib::ustring& layout_platform, const Gnome::Gda::Value& primary_key_value)
 {
-  //std::cout << "Box_Data_Details::init_db_details(): primary_key_value=" << primary_key_value.to_string() << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": primary_key_value=" << primary_key_value.to_string() << std::endl;
 
   m_primary_key_value = primary_key_value;
   m_field_primary_key = get_field_primary_key_for_table(found_set.m_table_name);
@@ -250,7 +245,7 @@ void Box_Data_Details::create_layout()
 
 bool Box_Data_Details::fill_from_database()
 {
-  //std::cout << "Box_Data_Details::fill_from_database(): m_primary_key_value=" << m_primary_key_value.to_string() << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": m_primary_key_value=" << m_primary_key_value.to_string() << std::endl;
 
   //Don't try to open a connection if there is no document,
   //for instance, during application destruction.
@@ -275,7 +270,6 @@ bool Box_Data_Details::fill_from_database()
   //reconnect many times..
   sharedptr<SharedConnection> sharedconnection;
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     sharedconnection = connect_to_server(get_app_window());
@@ -290,15 +284,6 @@ bool Box_Data_Details::fill_from_database()
     handle_error(ex);
     bResult = false;
   }
-#else
-  std::auto_ptr<ExceptionConnection> error;
-  sharedconnection = connect_to_server(get_app_window(), error);
-  if(error.get())
-  {
-    handle_error(*error);
-    bResult = false;
-  }
-#endif
 
   if(sharedconnection)
   {
@@ -306,7 +291,7 @@ bool Box_Data_Details::fill_from_database()
     bResult = Box_Data::fill_from_database();
 
     m_FieldsShown = get_fields_to_show();
-    type_vecLayoutFields fieldsToGet = m_FieldsShown;
+    type_vecConstLayoutFields fieldsToGet = m_FieldsShown;
 
     if(!fieldsToGet.empty())
     {
@@ -329,7 +314,7 @@ bool Box_Data_Details::fill_from_database()
         //TODO_Performance: Do this for create_layout() only, instead of repeating it for each refresh?:
         int index_primary_key = -1; //Arbitrary default.
         //g_warning("primary_key name = %s", m_field_primary_key->get_name().c_str());
-        const type_vecLayoutFields::const_iterator iterFind = std::find_if(fieldsToGet.begin(), fieldsToGet.end(), predicate_LayoutItem_Field_IsSameField<LayoutItem_Field>(layout_item));
+        const type_vecConstLayoutFields::const_iterator iterFind = std::find_if(fieldsToGet.begin(), fieldsToGet.end(), predicate_LayoutItem_Field_IsSameField<LayoutItem_Field>(layout_item));
         if(iterFind == fieldsToGet.end())
         {
           fieldsToGet.push_back(layout_item);
@@ -386,14 +371,7 @@ bool Box_Data_Details::fill_from_database()
             {
               if(index_primary_key < cols_count)
               {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
                 m_primary_key_value = result->get_value_at(index_primary_key, row_number);
-#else
-              {
-                std::auto_ptr<Glib::Error> value_error;
-                m_primary_key_value = result->get_value_at(index_primary_key, row_number, value_error);
-              }
-#endif
                 set_found_set_from_primary_key_value();
               }
             }
@@ -401,20 +379,13 @@ bool Box_Data_Details::fill_from_database()
             //Get field values to show:
             for(int i = 0; i < cols_count; ++i)
             {
-              sharedptr<LayoutItem_Field> layout_item = fieldsToGet[i];
+              sharedptr<const LayoutItem_Field> layout_item = fieldsToGet[i];
 
               //Field value:
               Gnome::Gda::Value value;
 
               if(!primary_key_is_empty)
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
                 value = result->get_value_at(i, row_number);
-#else
-              {
-                std::auto_ptr<Glib::Error> value_error;
-                value = result->get_value_at(i, row_number, value_error);
-              }
-#endif
               else
               {
                 value = Conversions::get_empty_value(layout_item->get_glom_type());
@@ -591,7 +562,7 @@ void Box_Data_Details::on_related_record_added(Gnome::Gda::Value /* strKeyValue
   bool bDoNotRefreshRelated = m_bDoNotRefreshRelated;
   m_bDoNotRefreshRelated = true;
 
-  //std::cout << "Box_Data_Details::on_related_record_added(): " << strKeyValue << ", " << strFromKeyName << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": " << strKeyValue << ", " << strFromKeyName << std::endl;
   //Get current FromKey value:
 
  /* TODO_port
@@ -690,28 +661,22 @@ void Box_Data_Details::on_flowtable_field_open_details_requested(const sharedptr
   if(Conversions::value_is_empty(field_value))
     return; //Ignore empty ID fields.
 
-  //Updating doesn't seem necessary. The field details seem to be full already.
-  //Update the field details from the document:
-  ////sharedptr<LayoutItem_Field> unconst_field = sharedptr<LayoutItem_Field>::cast_const(layout_field); //A hack, because layout_field_should_have_navigation() needs to get full field details.
-  //unconst_field->set_full_field_details(
-  //  document->get_field(field->get_table_used(table_name), field->get_name()) ); //Otherwise get_primary_key() returns false always.
-      
-  sharedptr<Relationship> field_used_in_relationship_to_one;
-  const bool has_open_button = 
-    DbUtils::layout_field_should_have_navigation(m_table_name, layout_field, get_document(), 
-    field_used_in_relationship_to_one);
-         
   //If it's a simple field that is part of a relationship,
   //identifying a related record.
-  if(field_used_in_relationship_to_one)
+  sharedptr<const Relationship> relationship = get_document()->get_field_used_in_relationship_to_one(m_table_name, layout_field);
+  if(relationship)
   {
-    signal_requested_related_details().emit(field_used_in_relationship_to_one->get_to_table(), field_value);
+    signal_requested_related_details().emit(relationship->get_to_table(), field_value);
     return;
   }
 
   //If it is a related field that is a primary key,
   //meaning it identifies a record in another table:
-  if(has_open_button)
+  sharedptr<const Field> field_info = layout_field->get_full_field_details();
+  const bool field_is_related_primary_key =
+    layout_field->get_has_relationship_name() &&
+    field_info && field_info->get_primary_key();
+  if(field_is_related_primary_key)
   {
     signal_requested_related_details().emit(layout_field->get_table_used(m_table_name), field_value);
   }
@@ -757,7 +722,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
   Document* document = dynamic_cast<Document*>(get_document());
 
   Gnome::Gda::Value primary_key_value = get_primary_key_value_selected();
-  //std::cout << "Box_Data_Details::on_flowtable_field_edited(): primary_key_value=" << primary_key_value.to_string() << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": primary_key_value=" << primary_key_value.to_string() << std::endl;
   if(!Conversions::value_is_empty(primary_key_value)) //If there is not a primary key value:
   {
     Glib::ustring table_name;
@@ -829,13 +794,10 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
     //Update the field in the record (the record with this primary key):
 
     bool bTest = false;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
-#endif // GLIBMM_EXCEPTIONS_ENABLED
     {
       bTest = set_field_value_in_database(field_in_record, field_value, false /* don't use current calculations */, get_app_window());
     }
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     catch(const Glib::Exception& ex)
     {
       handle_error(ex);
@@ -844,11 +806,8 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
     {
       handle_error(ex);
     }
-#endif // GLIBMM_EXCEPTIONS_ENABLED
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
-#endif // GLIBMM_EXCEPTIONS_ENABLED
     {
       if(!bTest)
       {
@@ -882,7 +841,6 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
       }
 
     }
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     catch(const Glib::Exception& ex)
     {
       handle_error(ex);
@@ -891,7 +849,6 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
     {
       handle_error(ex);
     }
-#endif // GLIBMM_EXCEPTIONS_ENABLED
   }
   else
   {
@@ -953,9 +910,8 @@ void Box_Data_Details::on_userlevel_changed(AppState::userlevels user_level)
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   m_design_mode = ( user_level == AppState::USERLEVEL_DEVELOPER );
   m_FlowTable.set_design_mode(m_design_mode);
-
   // Recreate the layout to correctly set the size of empty flowtables:
-  init_db_details(m_found_set, m_layout_platform, m_primary_key_value);
+  create_layout();
 #endif
 }
 
