@@ -26,9 +26,7 @@
 #include <libglom/privs.h>
 #include <libglom/db_utils.h>
 #include <libglom/utils.h> //For bold_message()).
-//#include <../utility_widgets/db_adddel/glom_db_treemodel.h> //For DbTreeModel.
 #include <sstream> //For stringstream
-#include <glibmm/main.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -95,7 +93,7 @@ void Box_Data_List::refresh_data_from_database_blank()
   found_set.m_where_clause = Gnome::Gda::SqlExpr();
   m_AddDel.set_found_set(found_set);
 
-  std::cout << "debug: Box_Data_List::refresh_data_from_database_blank(): before refresh_from_database_blank()." << std::endl;
+  std::cout << "debug: " << G_STRFUNC << ": before refresh_from_database_blank()." << std::endl;
   m_AddDel.refresh_from_database_blank();
   m_found_set = found_set;
 }
@@ -113,7 +111,6 @@ bool Box_Data_List::fill_from_database()
 
   sharedptr<SharedConnection> sharedconnection;
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     sharedconnection = connect_to_server(get_app_window());
@@ -128,15 +125,6 @@ bool Box_Data_List::fill_from_database()
     handle_error(ex);
     result = false;
   }
-#else
-  std::auto_ptr<ExceptionConnection> error;
-  sharedconnection = connect_to_server(get_app_window(), error);
-  if(error.get())
-  {
-    handle_error(*error);
-    result = false;
-  }
-#endif
 
   if(sharedconnection)
   {
@@ -289,7 +277,7 @@ void Box_Data_List::on_details_nav_next()
     //Don't go past the last record:
     if( !m_AddDel.get_is_last_row(iter) )
     {
-      std::cout << "DEBUG: Box_Data_List::on_details_nav_next(): The current row was not the last row." << std::endl;
+      std::cout << "debug: " << G_STRFUNC << ": The current row was not the last row." << std::endl;
 
       iter++;
       m_AddDel.select_item(iter);
@@ -297,7 +285,7 @@ void Box_Data_List::on_details_nav_next()
       signal_user_requested_details().emit(m_AddDel.get_value_key_selected());
     }
     else
-      std::cout << "DEBUG: Box_Data_List::on_details_nav_next(): Not going past the last row." << std::endl;
+      std::cout << "debug: " << G_STRFUNC << ": Not going past the last row." << std::endl;
   }
 }
 
@@ -359,7 +347,7 @@ Gnome::Gda::Value Box_Data_List::get_primary_key_value_selected() const
 
 Gnome::Gda::Value Box_Data_List::get_primary_key_value_first() const
 {
-  //std::cout << "Box_Data_List(): get_primary_key_value_first() records_count = " << m_AddDel.get_count() << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": get_primary_key_value_first() records_count = " << m_AddDel.get_count() << std::endl;
 
   Glib::RefPtr<Gtk::TreeModel> model = m_AddDel.get_model();
   if(model)
@@ -370,18 +358,18 @@ Gnome::Gda::Value Box_Data_List::get_primary_key_value_first() const
       Gnome::Gda::Value value = get_primary_key_value(iter);
       if(Conversions::value_is_empty(value))
       {
-       //std::cout << "Box_Data_List(): get_primary_key_value_first() iter val is NULL" << std::endl;
+       //std::cout << "debug: " << G_STRFUNC << ": get_primary_key_value_first() iter val is NULL" << std::endl;
         ++iter;
       }
       else
       {
-         //std::cout << "Box_Data_List(): get_primary_key_value_first() returning: " << value.to_string() << std::endl;
+         //std::cout << "debug: " << G_STRFUNC << ": get_primary_key_value_first() returning: " << value.to_string() << std::endl;
         return value;
       }
     }
   }
 
- // std::cout << "Box_Data_List(): get_primary_key_value_first() return NULL" << std::endl;
+ // std::cout << "debug: " << G_STRFUNC << ": get_primary_key_value_first() return NULL" << std::endl;
   return Gnome::Gda::Value();
 }
 
@@ -405,47 +393,6 @@ bool Box_Data_List::get_showing_multiple_records() const
   return m_AddDel.get_count() > 1;
 }
 
-void Box_Data_List::create_layout_add_group(const sharedptr<LayoutGroup>& layout_group)
-{
-  if(!layout_group)
-    return;
-
-  LayoutGroup::type_list_items child_items = layout_group->get_items();
-  for(LayoutGroup::type_list_items::const_iterator iter = child_items.begin(); iter != child_items.end(); ++iter)
-  {
-    sharedptr<LayoutItem> child_item = *iter;
-
-    sharedptr<LayoutGroup> child_group = sharedptr<LayoutGroup>::cast_dynamic(child_item);
-    if(child_group)
-    {
-      //std::cout << "debug: Start Adding child group." << std::endl;
-      create_layout_add_group(child_group);
-      //std::cout << "debug: End Adding child group." << std::endl;
-    }
-    else
-    {
-      if(m_read_only)
-        child_item->set_editable(false);
-
-      //std::cout << "debug: adding column: " << child_item->get_name() << std::endl;
-
-      sharedptr<LayoutItem_Field> child_field = sharedptr<LayoutItem_Field>::cast_dynamic(child_item);
-      if(child_field)
-      {
-        //Check that the field really exists, to avoid SQL errors.
-        //This could probably only happen if we have failed to rename something everywhere, when the user has renamed something.
-        if(!DbUtils::get_field_exists_in_database(child_field->get_table_used(m_table_name), child_field->get_name()))
-        {
-          std::cerr << "debug: Box_Data_List::create_layout_add_group(): Field does not exist in database: table_name=" << child_field->get_table_used(m_table_name) << ", field_name=" << child_field->get_name() << std::endl;
-          continue;
-        }
-      }
-
-      m_AddDel.add_column(child_item);
-    }
-  }
-}
-
 Document::type_list_layout_groups Box_Data_List::create_layout_get_layout()
 {
   //This method is overriden in Box_Data_List_Related.
@@ -458,62 +405,80 @@ void Box_Data_List::create_layout()
   Box_Data::create_layout(); //Fills m_TableFields.
 
   const Document* pDoc = dynamic_cast<const Document*>(get_document());
-  if(pDoc)
+  if(!pDoc)
+    return;
+
+
+  //Field Names:
+  m_AddDel.remove_all_columns();
+  //m_AddDel.set_columns_count(m_Fields.size());
+
+  m_AddDel.set_table_name(m_table_name);
+
+
+  sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
+  if(!field_primary_key)
   {
-    //Field Names:
-    m_AddDel.remove_all_columns();
-    //m_AddDel.set_columns_count(m_Fields.size());
-
-    m_AddDel.set_table_name(m_table_name);
-
-
-    sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
-    if(!field_primary_key)
-    {
-      std::cerr << "Box_Data_List::create_layout(): primary key not found." << std::endl;
-    }
-    else
-    {
-      //std::cout << "DEBUG: Box_Data_List::create_layout(): primary_key=" << field_primary_key->get_name() << std::endl;
-
-      m_AddDel.set_key_field(field_primary_key);
-
-      //This map of layout groups will also contain the field information from the database:
-      Document::type_list_layout_groups layout_groups = create_layout_get_layout();
-
-      //int debug_count = 0;
-      for(Document::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
-      {
-        //std::cout << "Box_Data_List::create_layout() group number=" << debug_count;
-        //debug_count++;
-        //iter->second->debug();
-
-        create_layout_add_group(*iter);
-      }
-    }
-
-
-    m_FieldsShown = get_fields_to_show();
-
-    //Add extra possibly-non-visible columns that we need:
-    //TODO: Only add it if it is not already there.
-    if(field_primary_key)
-    {
-      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
-      layout_item->set_hidden();
-      layout_item->set_full_field_details(m_AddDel.get_key_field());
-      m_FieldsShown.push_back(layout_item);
-
-      m_AddDel.add_column(layout_item);
-    }
-
-    m_AddDel.set_found_set(m_found_set);
-
-    //Column-creation happens in fill_database() instead:
-    //otherwise the treeview will be filled twice.
-    //m_AddDel.set_columns_ready();
+    std::cerr << G_STRFUNC << ": primary key not found." << std::endl;
+    return;
   }
 
+   m_AddDel.set_key_field(field_primary_key);
+
+
+
+  LayoutGroup::type_list_items items_to_use;
+
+  //This map of layout groups will also contain the field information from the database:
+  Document::type_list_layout_groups layout_groups = create_layout_get_layout();
+  for(Document::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
+  {
+    const sharedptr<LayoutGroup> layout_group = *iter;
+    if(!layout_group)
+      continue;
+
+    const LayoutGroup::type_list_items child_items = layout_group->get_items_recursive();
+    for(LayoutGroup::type_list_items::const_iterator iterItems = child_items.begin(); iterItems != child_items.end(); ++iterItems)
+    {
+      sharedptr<LayoutItem> child_item = *iterItems;
+
+      //TODO: Set the whole thing as read-only instead:
+      if(m_read_only)
+        child_item->set_editable(false);
+
+      sharedptr<const LayoutItem_Field> child_field = sharedptr<const LayoutItem_Field>::cast_dynamic(child_item);
+      if(child_field)
+      {
+        //Check that the field really exists, to avoid SQL errors.
+        //This could probably only happen if we have failed to rename something everywhere, when the user has renamed something.
+        if(!DbUtils::get_field_exists_in_database(child_field->get_table_used(m_table_name), child_field->get_name()))
+        {
+          std::cerr << G_STRFUNC << ": Field does not exist in database: table_name=" << child_field->get_table_used(m_table_name) << ", field_name=" << child_field->get_name() << std::endl;
+          continue;
+        }
+      }
+
+      items_to_use.push_back(child_item);
+    }
+  }
+
+
+  //Add extra possibly-non-visible columns that we need:
+  //TODO: Only add it if it is not already there.
+  if(field_primary_key)
+  {
+    sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+    layout_item->set_hidden();
+    layout_item->set_full_field_details(m_AddDel.get_key_field());
+    m_FieldsShown.push_back(layout_item);
+
+    items_to_use.push_back(layout_item);
+  }
+
+  m_AddDel.set_found_set(m_found_set);
+  m_AddDel.set_columns(items_to_use);
+
+  m_FieldsShown = get_fields_to_show();
 }
 
 sharedptr<Field> Box_Data_List::get_field_primary_key() const
@@ -538,9 +503,11 @@ void Box_Data_List::print_layout() //TODO: Is this a duplicate of the one in box
     report_temp->set_title(_("List"));
 
     //Add all the fields from the layout:
-    for(type_vecLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
+    for(type_vecConstLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
     {
-      report_temp->m_layout_group->add_item(*iter);
+      sharedptr<const LayoutItem> item = *iter;
+      sharedptr<LayoutItem> unconst = sharedptr<LayoutItem>::cast_const(item); //TODO: Avoid this?
+      report_temp->m_layout_group->add_item(unconst);
     }
 
     ReportBuilder report_builder;
