@@ -18,8 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef NOTEBOOK_DATA_H
-#define NOTEBOOK_DATA_H
+#ifndef GLOM_MODE_DATA_NOTEBOOK_DATA_H
+#define GLOM_MODE_DATA_NOTEBOOK_DATA_H
 
 #include <glom/notebook_glom.h>
 #include <glom/mode_data/box_data_list.h>
@@ -88,7 +88,7 @@ protected:
   virtual void on_list_user_requested_details(const Gnome::Gda::Value& primary_key_value);
   void on_details_user_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value);
 
-  virtual void on_switch_page_handler(GtkNotebookPage* pPage, guint uiPageNumber);
+  virtual void on_switch_page_handler(Gtk::Widget* pPage, guint uiPageNumber);
 
 private:
   #ifdef GLOM_ENABLE_MAEMO
@@ -114,4 +114,4 @@ protected:
 
 } //namespace Glom
 
-#endif
+#endif // GLOM_MODE_DATA_NOTEBOOK_DATA_H
