@@ -143,7 +143,7 @@ void Dialog_Database_Preferences::on_treeview_cell_edited_next_value(const Glib:
 
     const bool test = DbUtils::query_execute(builder);
     if(!test)
-      std::cerr << "Dialog_Database_Preferences::on_treeview_cell_edited_next_value(): UPDATE failed." << std::endl;
+      std::cerr << G_STRFUNC << ": UPDATE failed." << std::endl;
   }
 }
 
@@ -189,7 +189,7 @@ void Dialog_Database_Preferences::load_from_document()
   Glib::RefPtr<Gnome::Gda::DataModel> datamodel = DbUtils::query_execute_select(builder);
   if(!datamodel)
   {
-    std::cerr << "Dialog_Database_Preferences::load_from_document(): Gda::DataModel is NULL." << std::endl;
+    std::cerr << G_STRFUNC << ": Gda::DataModel is NULL." << std::endl;
     return;
   }
 
@@ -282,11 +282,6 @@ void Dialog_Database_Preferences::on_button_test_script()
     sharedconnection->get_gda_connection(),
     callbacks,
     error_message);
-
-  if(!error_message.empty())
-  {
-    std::cerr << "Python Error: " << error_message << std::endl;
-  }
 }
 
 
