@@ -18,12 +18,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef DIALOG_FIELDCALCULATION_H
-#define DIALOG_FIELDCALCULATION_H
+#ifndef GLOM_MODE_DESIGN_DIALOG_FIELDCALCULATION_H
+#define GLOM_MODE_DESIGN_DIALOG_FIELDCALCULATION_H
 
-#include <gtkmm/dialog.h>
-#include <gtkmm/button.h>
-#include <gtkmm/label.h>
+#include <gtkmm.h>
 #include <gtkmm/builder.h>
 #include <libglom/data_structure/field.h>
 #include <glom/base_db.h>
@@ -61,4 +59,4 @@ private:
 
 } //namespace Glom
 
-#endif //DIALOG_FIELDCALCULATION_H
+#endif // GLOM_MODE_DESIGN_DIALOG_FIELDCALCULATION_H
