@@ -56,7 +56,7 @@ Dialog_Layout_Calendar_Related::Dialog_Layout_Calendar_Related(BaseObjectType* c
   builder->get_widget("checkbutton_show_child_relationships", m_checkbutton_show_child_relationships);
   m_checkbutton_show_child_relationships->signal_toggled().connect(sigc::mem_fun(*this, &Dialog_Layout_Calendar_Related::on_checkbutton_show_child_relationships));
 
-  
+
 
   builder->get_widget("radiobutton_navigation_automatic", m_radio_navigation_automatic);
   builder->get_widget("label_navigation_automatic", m_label_navigation_automatic);
@@ -70,7 +70,7 @@ Dialog_Layout_Calendar_Related::Dialog_Layout_Calendar_Related(BaseObjectType* c
   builder->get_widget_derived("combobox_date_field", m_combobox_date_field);
   m_combobox_date_field->signal_changed().connect(sigc::mem_fun(*this, &Dialog_Layout_Calendar_Related::on_combo_date_field_changed));
 
-  
+
   m_modified = false;
 
   //show_all_children();
@@ -78,9 +78,9 @@ Dialog_Layout_Calendar_Related::Dialog_Layout_Calendar_Related(BaseObjectType* c
   //The base class hid this, but we do want it here:
   //(We share one glade definition for several dialogs.)
   Gtk::Frame* box_calendar = 0;
-  builder->get_widget("frame_calendar", box_calendar); 
+  builder->get_widget("frame_calendar", box_calendar);
   box_calendar->show();
-  
+
   //This entry must be in the Glade file, because it's used by the base class,
   //but we don't want it here, because it is confusing when dealing with relationships:
   if(m_entry_table_title)
@@ -98,7 +98,7 @@ Dialog_Layout_Calendar_Related::~Dialog_Layout_Calendar_Related()
 void Dialog_Layout_Calendar_Related::set_document(const Glib::ustring& layout, const Glib::ustring& layout_platform, Document* document, const sharedptr<const LayoutItem_CalendarPortal>& portal)
 {
   m_portal = glom_sharedptr_clone(portal);
-  
+
   Glib::ustring from_table;
   if(portal)
     from_table = portal->get_from_table();
@@ -113,9 +113,9 @@ void Dialog_Layout_Calendar_Related::set_document(const Glib::ustring& layout_na
     m_portal = sharedptr<LayoutItem_CalendarPortal>::create(); //The rest of the class assumes that this is not null.
   }
 
-  type_vecLayoutFields empty_fields; //Just to satisfy the base class.
+  type_vecConstLayoutFields empty_fields; //Just to satisfy the base class.
+
 
-  
   Dialog_Layout::set_document(layout_name, layout_platform, document, from_table, empty_fields);
   //m_table_name is now actually the parent_table_name.
 
@@ -136,7 +136,7 @@ void Dialog_Layout_Calendar_Related::update_ui(bool including_relationship_list)
     if(including_relationship_list)
     {
       bool show_child_relationships = m_checkbutton_show_child_relationships->get_active();
-        
+
       //For the showing of child relationships if necessary:
       if(!show_child_relationships && m_portal && m_portal->get_related_relationship())
       {
@@ -159,7 +159,7 @@ void Dialog_Layout_Calendar_Related::update_ui(bool including_relationship_list)
 
     //Set the table name and title:
     //sharedptr<LayoutItem_CalendarPortal> portal_temp = m_portal;
-    m_combo_relationship->set_selected_relationship(m_portal->get_relationship(), m_portal->get_related_relationship()); 
+    m_combo_relationship->set_selected_relationship(m_portal->get_relationship(), m_portal->get_related_relationship());
 
     Document::type_list_layout_groups mapGroups;
     if(m_portal)
@@ -220,16 +220,19 @@ void Dialog_Layout_Calendar_Related::update_ui(bool including_relationship_list)
 
   //Set the appropriate radio button:
   //std::cout << "debug: navigation_is_automatic=" << navigation_is_automatic << std::endl;
-  m_radio_navigation_automatic->set_active(navigation_is_automatic); 
-  m_radio_navigation_specify->set_active(!navigation_is_automatic); 
+  m_radio_navigation_automatic->set_active(navigation_is_automatic);
+  m_radio_navigation_specify->set_active(!navigation_is_automatic);
 
 
   //Describe the automatic navigation:
-  sharedptr<const UsesRelationship> relationship_navigation_automatic
-    = m_portal->get_portal_navigation_relationship_automatic(document);
-  Glib::ustring automatic_navigation_description = 
-    m_portal->get_relationship_name_used(); //TODO: Use get_relationship_display_name() instead?
-  if(relationship_navigation_automatic) //This is a relationship in the related table.
+  sharedptr<const UsesRelationship> relationship_navigation_automatic;
+  bool navigation_automatic_main = false;
+  relationship_navigation_automatic = get_portal_navigation_relationship_automatic(m_portal, navigation_automatic_main);
+  Glib::ustring automatic_navigation_description;
+
+  if(navigation_automatic_main)
+    automatic_navigation_description = m_portal->get_relationship_name_used();
+  else if(relationship_navigation_automatic)
   {
     automatic_navigation_description = m_portal->get_relationship_name_used();
 
@@ -249,13 +252,13 @@ void Dialog_Layout_Calendar_Related::update_ui(bool including_relationship_list)
 
   sharedptr<Field> debugfield = m_portal->get_date_field();
   if(!debugfield)
-    std::cout << "DEBUG: set_document(): date field is NULL" << std::endl;
+    std::cout << "debug: " << G_STRFUNC << ": date field is NULL" << std::endl;
   else
-    std::cout << "DEBUG: set_document(): date field:" << debugfield->get_name() << std::endl;
-  
+    std::cout << "debug: " << G_STRFUNC << ": date field:" << debugfield->get_name() << std::endl;
+
   m_combobox_date_field->set_fields(document, related_table_name, Field::TYPE_DATE);
   m_combobox_date_field->set_selected_field(m_portal->get_date_field());
-    
+
   m_modified = false;
 }
 
@@ -308,14 +311,14 @@ void Dialog_Layout_Calendar_Related::save_to_document()
       sharedptr<UsesRelationship> none;
       m_portal->set_navigation_relationship_specific(none);
     }
-    
+
     m_portal->set_date_field( m_combobox_date_field->get_selected_field() );
-    
+
     sharedptr<Field> debugfield = m_portal->get_date_field();
     if(!debugfield)
-      std::cout << "DEBUG: save_to_document(): date field is NULL" << std::endl;
+      std::cout << "debug: " << G_STRFUNC << ": date field is NULL" << std::endl;
     else
-      std::cout << "DEBUG: save_to_document(): date field:" << debugfield->get_name() << std::endl;
+      std::cout << "debug: " << G_STRFUNC << ": date field:" << debugfield->get_name() << std::endl;
   }
 }
 
@@ -378,7 +381,7 @@ void Dialog_Layout_Calendar_Related::on_button_add_field()
   //std::cout << "debug table used =" << m_portal->get_table_used(m_table_name) << std::endl;
 
   type_list_field_items fields_list = offer_field_list(m_table_name, this);
-  for(type_list_field_items::iterator iter_chosen = fields_list.begin(); iter_chosen != fields_list.end(); ++iter_chosen) 
+  for(type_list_field_items::iterator iter_chosen = fields_list.begin(); iter_chosen != fields_list.end(); ++iter_chosen)
   {
     sharedptr<LayoutItem_Field> field = *iter_chosen;
     if(!field)
@@ -444,4 +447,3 @@ sharedptr<LayoutItem_CalendarPortal> Dialog_Layout_Calendar_Related::get_portal_
 
 
 } //namespace Glom
-
