@@ -18,8 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef BOX_DATA_LIST_FIND_H
-#define BOX_DATA_LIST_FIND_H
+#ifndef GLOM_BOX_DATA_LIST_FIND_H
+#define GLOM_BOX_DATA_LIST_FIND_H
 
 #include "../mode_data/box_data_list.h"
 
@@ -36,7 +36,7 @@ public:
 
   virtual Gtk::Widget* get_default_button(); //override
 
-protected:
+private:
 
   virtual bool fill_from_database(); //override.
   virtual void create_layout();
@@ -47,4 +47,4 @@ protected:
 
 } //namespace Glom
 
-#endif
+#endif // GLOM_BOX_DATA_LIST_FIND_H
