@@ -229,7 +229,7 @@ static boost::python::object glom_python_call(Field::glom_field_type result_type
   //boost::python::dict pDict = pMain.attr("__dict__"); //TODO: Does boost::python have an equivalent for PyModule_GetDict()?
   if(!pDict)
   {
-     std::cerr << "glom_evaluate_python_function_implementation(): pDict is null" << std::endl;
+     std::cerr << G_STRFUNC << ": pDict is null" << std::endl;
      error_message = get_traceback();
      return boost::python::object();
   }
@@ -303,14 +303,14 @@ static boost::python::object glom_python_call(Field::glom_field_type result_type
   }
   catch(const boost::python::error_already_set& ex)
   {
-    std::cerr << "glom_evaluate_python_function_implementation():  boost::python::exec() threw error_already_set when using text= " << std::endl << func_def << std::endl;
+    std::cerr << G_STRFUNC << ":  boost::python::exec() threw error_already_set when using text= " << std::endl << func_def << std::endl;
     error_message = get_traceback();
     return boost::python::object();
   }
 
   if(!pyValue.ptr())
   {
-    std::cerr << "glom_evaluate_python_function_implementation(): boost::python::exec failed." << std::endl;
+    std::cerr << G_STRFUNC << ": boost::python::exec failed." << std::endl;
     error_message = get_traceback();
     return boost::python::object();
   }
@@ -323,14 +323,14 @@ static boost::python::object glom_python_call(Field::glom_field_type result_type
   }
   catch(const boost::python::error_already_set& ex)
   {
-    std::cerr << "glom_evaluate_python_function_implementation():  pDict[func_name] threw error_already_set when func_name= " << std::endl << func_name << std::endl;
+    std::cerr << G_STRFUNC << ":  pDict[func_name] threw error_already_set when func_name= " << std::endl << func_name << std::endl;
     error_message = get_traceback();
     return boost::python::object();
   }
 
   if(!pFunc.ptr())
   {
-    std::cerr << "glom_evaluate_python_function_implementation(): pDict[func_name] failed." << std::endl;
+    std::cerr << G_STRFUNC << ": pDict[func_name] failed." << std::endl;
     HandlePythonError();
     return boost::python::object();
   }
@@ -465,20 +465,9 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
   if(object_is_gda_value && G_IS_VALUE(&value))
   {
     valueResult = Gnome::Gda::Value(&value);
-    if(valueResult.get_value_type() == 0)
-    {
-      std::cerr << G_STRFUNC << ": valueResult (before convert_value()) has a GType of 0 before convert_value()." << std::endl;
-    }
-
     //Make sure that the value is of the expected Gda type:
     //TODO_Performance:
     valueResult = Glom::Conversions::convert_value(valueResult, result_type);
-
-    if(valueResult.get_value_type() == 0)
-    {
-      std::cerr << G_STRFUNC << ": valueResult has a GType of 0 after convert_value()." << std::endl;
-    }
-
     //std::cout << "debug: " << G_STRFUNC << ": valueResult Gda type=" << g_type_name(valueResult.get_value_type()) << std::endl;
     g_value_unset(&value);
   }
@@ -492,7 +481,7 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
     {
       //The result should be an appropriate empty value for this field type:
       valueResult = Conversions::get_empty_value(result_type);
-      //std::cout << "DEBUG: glom_evaluate_python_function_implementation(): empty value Gda type=" << g_type_name(valueResult.get_value_type()) << std::endl;
+      //std::cout << "debug: " << G_STRFUNC << ": empty value Gda type=" << g_type_name(valueResult.get_value_type()) << std::endl;
     }
     else
     {
@@ -516,7 +505,7 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
       {
         bool success = false;
         valueResult = Conversions::parse_value(result_type, pchResult, success, true /* iso_format */);
-        std::cout << "DEBUG: glom_evaluate_python_function_implementation(): parsed value Gda type=" << g_type_name(valueResult.get_value_type()) << std::endl;
+        std::cout << "debug: " << G_STRFUNC << ": parsed value Gda type=" << g_type_name(valueResult.get_value_type()) << std::endl;
       }
       else
         HandlePythonError();
