@@ -23,12 +23,11 @@
 #include <glom/utility_widgets/adddel/adddel.h>
 #include <algorithm> //For std::find.
 #include <glibmm/i18n.h>
-#include <glom/utility_widgets/cellrendererlist/cellrendererlist.h>
+#include <glom/utility_widgets/cellrendererlist.h>
 #include <glom/utility_widgets/adddel/treeviewcolumn_glom.h>
 #include <libglom/data_structure/glomconversions.h>
 #include <glom/dialog_invalid_data.h>
 #include <glom/utils_ui.h>
-#include <gtkmm/stock.h>
 #include <libglom/utils.h>
 //#include <glom/bakery/app_gtk.h>
 #include <iostream> //For debug output.
@@ -149,7 +148,7 @@ void AddDel::set_treeview_accessible_name(const Glib::ustring& name)
 {
 #ifdef GTKMM_ATKMM_ENABLED
   m_TreeView.get_accessible()->set_name(name);
-#endif
+#endif  
 }
 
 void AddDel::warn_about_duplicate()
@@ -255,11 +254,9 @@ void AddDel::setup_menu()
 
   //TODO: add_accel_group(m_refUIManager->get_accel_group());
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-#endif
-    Glib::ustring ui_info =
+    Glib::ustring ui_info = 
         "<ui>"
         "  <popup name='ContextMenu'>"
         "    <menuitem action='ContextEdit'/>"
@@ -267,24 +264,15 @@ void AddDel::setup_menu()
         "  </popup>"
         "</ui>";
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     m_refUIManager->add_ui_from_string(ui_info);
   }
   catch(const Glib::Error& ex)
   {
     std::cerr << "building menus failed: " <<  ex.what();
   }
-#else
-  std::auto_ptr<Glib::Error> error;
-  m_refUIManager->add_ui_from_string(ui_info, error);
-  if(error.get())
-  {
-    std::cerr << "building menus failed: " << error->what();
-  }
-#endif //GLIBMM_EXCEPTIONS_ENABLED
 
   //Get the menu:
-  m_pMenuPopup = dynamic_cast<Gtk::Menu*>( m_refUIManager->get_widget("/ContextMenu") );
+  m_pMenuPopup = dynamic_cast<Gtk::Menu*>( m_refUIManager->get_widget("/ContextMenu") ); 
   if(!m_pMenuPopup)
     g_warning("menu not found");
 }
@@ -323,7 +311,7 @@ Gtk::TreeModel::iterator AddDel::get_item_placeholder()
     return add_item_placeholder();
   }
 }
-
+  
 Gtk::TreeModel::iterator AddDel::add_item_placeholder()
 {
   Gtk::TreeModel::iterator iter = m_refListStore->append();
@@ -405,7 +393,7 @@ bool AddDel::get_value_as_bool(const Gtk::TreeModel::iterator& iter, guint col)
 {
 //TODO: I doubt that this works. It should really get the value from the treeview as a bool. murrayc
   Glib::ustring strValue = get_value(iter, col);
-  return (strValue == "true");
+  return (strValue == "true");   
 }
 
 Glib::ustring AddDel::get_value_key_selected()
@@ -527,7 +515,7 @@ void AddDel::add_blank()
 
   if(get_allow_user_actions()) //The extra blank line is only used if the user may add items:
   {
-    Gtk::TreeModel::iterator iter = get_last_row();
+    Gtk::TreeModel::iterator iter = get_last_row();        
     if(get_is_placeholder_row(iter))
     {
         bAddNewBlank  = false; //One already exists.
@@ -770,9 +758,9 @@ void AddDel::construct_specified_columns()
     }
     else
     {
-      std::cerr << "debug: AddDel::construct_specified_columns(): Leaking a Gtk::TreeModelColumn<>." << std::endl;
+      std::cerr << G_STRFUNC << ": Leaking a Gtk::TreeModelColumn<>." << std::endl;
     }
-
+    
     ++model_column_index;
   }
 
@@ -912,7 +900,7 @@ guint AddDel::add_column(const AddDelColumnInfo& column_info)
 
   return m_ColumnTypes.size()-1;
 }
-
+ 
 guint AddDel::add_column(const Glib::ustring& strTitle, AddDelColumnInfo::enumStyles style, bool editable, bool visible)
 {
   //Use the title as the column_id:
@@ -989,8 +977,8 @@ void AddDel::set_allow_delete(bool val)
 {
   m_allow_delete= val;
 }
-
-
+  
+  
 void AddDel::set_allow_user_actions(bool bVal)
 {
   m_bAllowUserActions = bVal;
@@ -1097,7 +1085,7 @@ Glib::ustring AddDel::treeview_get_key(const Gtk::TreeModel::iterator& row)
 
   if(m_refListStore)
   {
-    Gtk::TreeModel::Row treerow = *row;
+    Gtk::TreeModel::Row treerow = m_refListStore->children()[row];
 
     if(treerow)
       treerow.get_value(m_col_key, value);
@@ -1127,7 +1115,7 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
     row.set_value(model_column_index, value_new);
 
     //TODO: Did it really change?
-
+    
     //Is this an add or a change?:
 
     bool bIsAdd = false;
@@ -1166,7 +1154,7 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
     {
       //Change it back, so that we ignore it:
       row.set_value(model_column_index, value_old);
-
+         
       //Signal that a new key was added:
       //We will ignore editing of bool values in the blank row. It seems like a bad way to start a new record.
       //m_signal_user_added.emit(row);
@@ -1177,7 +1165,7 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
 
       m_signal_user_changed.emit(row, model_column_index);
     }
-
+    
   }
 }
 
@@ -1239,7 +1227,7 @@ void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Gli
     //Fire appropriate signal:
     if(bIsAdd)
     {
-        //Signal that a new key was added"
+        //Signal that a new key was added"        
         m_signal_user_added.emit(row);
     }
     else if(bIsChange)
@@ -1522,7 +1510,7 @@ void AddDel::set_value_key(const Gtk::TreeModel::iterator& iter, const Glib::ust
     //This is not a placeholder anymore, if it every was:
     iter->set_value(m_col_placeholder, false);
   }
-
+  
   iter->set_value(m_col_key, strValue);
 }
 
@@ -1552,7 +1540,7 @@ bool AddDel::get_model_column_index(guint view_column_index, guint& model_column
   const guint count = m_ColumnTypes.size();
   if(hidden > count)
     return false; //This should never happen.
-
+  
   if(view_column_index >=  (count - hidden))
     return false;
 
