@@ -24,12 +24,8 @@
 #include <glom/utils_ui.h>
 #include <glom/glade_utils.h>
 #include <libglom/data_structure/glomconversions.h>
-#include <gtkmm/filechooserdialog.h>
-#include <gtkmm/stock.h>
-#include <giomm.h>
-#include <giomm/file.h>
-#include <giomm/contenttype.h>
-#include <glibmm/convert.h>
+//#include <sstream> //For stringstream
+
 #include <iostream>   // for cout, endl
 
 namespace Glom
@@ -58,11 +54,6 @@ void ImageGlom::init()
   setup_menu();
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-#ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
-  signal_button_press_event().connect(sigc::mem_fun(*this, &ImageGlom::on_button_press_event), false);
-  signal_expose_event().connect(sigc::mem_fun(*this, &ImageGlom::on_expose_event));
-#endif
-
   setup_menu_usermode();
 
   m_image.set_size_request(150, 150);
@@ -141,11 +132,7 @@ bool ImageGlom::on_button_press_event(GdkEventButton *event)
     }
   }
 
-#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   return Gtk::EventBox::on_button_press_event(event);
-#else
-  return false;
-#endif // GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
 }
 
 Application* ImageGlom::get_application()
@@ -242,23 +229,16 @@ Gnome::Gda::Value ImageGlom::get_value() const
   
   if(m_pixbuf_original)
   {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
-#endif
     {
       gchar* buffer = 0;
       gsize buffer_size = 0;
-      std::vector<Glib::ustring> list_keys;
-      std::vector<Glib::ustring> list_values;
+      std::list<Glib::ustring> list_keys;
+      std::list<Glib::ustring> list_values;
       //list_keys.push_back("quality"); //For jpeg only.
       //list_values.push_back("95");
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
       m_pixbuf_original->save_to_buffer(buffer, buffer_size, GLOM_IMAGE_FORMAT, list_keys, list_values); //Always store images as the standard format in the database.
-#else
-      std::auto_ptr<Glib::Error> error;
-      m_pixbuf_original->save_to_buffer(buffer, buffer_size, GLOM_IMAGE_FORMAT, list_keys, list_values, error); //Always store images as the standard format in the database.
-#endif
 
       //g_warning("ImageGlom::get_value(): debug: to db: ");
       //for(int i = 0; i < 10; ++i)
@@ -275,26 +255,19 @@ Gnome::Gda::Value ImageGlom::get_value() const
       buffer = 0;
       return m_original_data;
     }
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     catch(const Glib::Exception& ex)
     {
-      std::cerr << "ImageGlom::get_value(): " << ex.what() << std::endl;
+      std::cerr << G_STRFUNC << ": " << ex.what() << std::endl;
     }
-#endif // GLIBMM_EXCEPTIONS_ENABLED
   }
 
   return Gnome::Gda::Value();
 }
 
-#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
-bool ImageGlom::on_expose_event(GdkEventExpose* event)
+bool ImageGlom::on_draw(const Cairo::RefPtr<Cairo::Context>& cr)
 {
-  const bool result = Gtk::EventBox::on_expose_event(event);
-#else
-bool ImageGlom::on_expose_event(GdkEventExpose* /* event */)
-{
-  const bool result = false;
-#endif // GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
+  const bool result = Gtk::EventBox::on_draw(cr);
+
   scale();
   return result;
 }
@@ -346,9 +319,9 @@ void ImageGlom::on_menupopup_activate_select_file()
   Gtk::FileChooserDialog dialog(_("Choose Image"), Gtk::FILE_CHOOSER_ACTION_OPEN);
 
   //Get image formats only:
-  Gtk::FileFilter filter;
-  filter.set_name(_("Images"));
-  filter.add_pixbuf_formats();
+  Glib::RefPtr<Gtk::FileFilter> filter = Gtk::FileFilter::create();
+  filter->set_name(_("Images"));
+  filter->add_pixbuf_formats();
   dialog.add_filter(filter);
 
   dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
@@ -382,9 +355,7 @@ void ImageGlom::on_menupopup_activate_select_file()
           bin->binary_length = image_data->binary_length;
 
           m_original_data = Gnome::Gda::Value();
-          
-          g_value_unset(m_original_data.gobj());
-          g_value_init(m_original_data.gobj(), GDA_TYPE_BINARY);
+          m_original_data.Glib::ValueBase::init(GDA_TYPE_BINARY);
           gda_value_take_binary(m_original_data.gobj(), bin);
 
           m_pixbuf_original = dialog->get_pixbuf();
@@ -436,7 +407,8 @@ void ImageGlom::on_menupopup_activate_copy()
   Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
 
   //Targets:
-  std::vector<Gtk::TargetEntry> listTargets;
+  std::list<Gtk::TargetEntry> listTargets;
+
   listTargets.push_back( Gtk::TargetEntry(GLOM_IMAGE_FORMAT_MIME_TYPE) );
 
   refClipboard->set( listTargets, sigc::mem_fun(*this, &ImageGlom::on_clipboard_get), sigc::mem_fun(*this, &ImageGlom::on_clipboard_clear) );
@@ -511,11 +483,7 @@ void ImageGlom::setup_menu_usermode()
 
   //TODO: add_accel_group(m_refUIManager_UserModePopup->get_accel_group());
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
-#else
-  std::auto_ptr<Glib::Error> error;
-#endif // GLIBMM_EXCEPTIONS_ENABLED
   {
     Glib::ustring ui_info = 
         "<ui>"
@@ -527,20 +495,10 @@ void ImageGlom::setup_menu_usermode()
         "  </popup>"
         "</ui>";
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     m_refUIManager_UserModePopup->add_ui_from_string(ui_info);
-#else
-    m_refUIManager_UserModePopup->add_ui_from_string(ui_info, error);
-#endif // GLIBMM_EXCEPTIONS_ENABLED
   }
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   catch(const Glib::Error& ex)
   {
-#else
-  if(error.get())
-  {
-    const Glib::Error& ex = *error.get();
-#endif
     std::cerr << "building menus failed: " <<  ex.what();
   }
 
